package com.visutools.nav.bislider;

import java.awt.AWTException;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Robot;
import java.awt.Shape;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.Popup;
import javax.swing.PopupFactory;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * The graphical interface of the bean (drawing and mouse event handling). <br>
 * <br>
 * <table border=1 width="90%">
 * <tr>
 * <td> Copyright 1997-2011 Frederic Vernier. All Rights Reserved.<br>
 * <br>
 * Permission to use, copy, modify and distribute this software and its
 * documentation for educational, research and non-profit purposes, without fee,
 * and without a written agreement is hereby granted, provided that the above
 * copyright notice and the following three paragraphs appear in all copies.<br>
 * <br>
 * To request Permission to incorporate this software into commercial products
 * contact Frederic Vernier, 19 butte aux cailles street, Paris, 75013, France.
 * Tel: (+33) 871 747 387. eMail: Frederic.Vernier@laposte.net / Web site:
 * http://vernier.frederic.free.fr <br>
 * IN NO EVENT SHALL FREDERIC VERNIER BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST
 * PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF FREDERIC VERNIER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.<br>
 * <br>
 * FREDERIC VERNIER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HERE UNDER IS ON AN "AS IS" BASIS,
 * AND FREDERIC VERNIER HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.<br>
 * </td>
 * </tr>
 * </table> <br>
 * <b>Project related :</b> FiCell, FieldExplorer<br>
 * <br>
 * <b>Dates:</b> <br>
 * <li> Format : 01/11/2001
 * <li> Last Modif : 27/04/2011 <br>
 * <b>Bugs:</b> <br>
 * <li> ??? <br>
 * <b>To Do:</b> <br>
 * <li> alt or shift pressed -> repaint
 * 
 * @author Frederic Vernier, Frederic.Vernier@laposte.net
 * @version 1.4.1
 * @created 16 fevrier 2004
 */
public class BiSliderPresentation extends ComponentUI implements Serializable,
        MouseListener, MouseMotionListener, MouseWheelListener,
        ComponentListener {
  // ------------ MODIFIERS|--------------------
  // Type|----------------------------------------------- Name = Init value
  protected final static javax.swing.text.html.parser.ParserDelegator MAXIMUM_VARIABLE_SIZE_FOR_NAME = null;
  final static long serialVersionUID = 6420110040552724883L;
  protected BiSlider Ctrl = null;
  protected JComponent JComponent1 = null;
  protected int[][] ColorTable = null;
  protected long j = 0;
  protected int NbAff = 0;

  // for feedback
  protected final int NOTHING = 200;
  protected final int RIGHT_POLYGON = 201;
  protected final int LEFT_POLYGON = 202;
  protected final int SELECTION = 203;
  protected final int FIRST_LABEL = 204;
  protected final int LAST_LABEL = 205;
  protected int MouseUnder = this.NOTHING;

  // Triangle polygons for the rulers
  protected Polygon TheLeft_Polygon = null;
  protected Polygon TheRight_Polygon = null;
  protected Rectangle2D RectFirstLabel = null;
  protected Rectangle2D RectLastLabel = null;
  protected Rectangle2D RectangleSegment = null;

  // indicate if the tool has already been painted on screen or never.
  protected boolean NeverDrawn = true;

  // graduation variables
  protected double GraduationWidth = 1;

  // we need to remember the precise value opened when the precise popup shows
  // up because we will move around it
  protected double PreciseOpenedValue = 0;

  // Constants for the Dragging variable
  protected final static int NONE = 100;
  protected final static int LEFT_RULER = 101;
  protected final static int RIGHT_RULER = 102;
  protected final static int SHIFT_LEFT_RULER = 103;
  protected final static int SHIFT_RIGHT_RULER = 104;
  protected final static int CTRL_LEFT_RULER = 105;
  protected final static int CTRL_RIGHT_RULER = 106;
  
  protected final static int SEGMENT = 107;
  protected final static int SHIFT_SEGMENT = 108;
  protected final static int CTRL_SEGMENT = 109;  
  protected final static int ALT_LEFT_RULER = 110;
  protected final static int ALT_RIGHT_RULER = 111;
  protected final static int SEGMENT_SIZE = 112;
  protected final static int SEGMENT_SIZE_INT = 113;
  protected final static int PRECISE_LEFT_RULER = 114;
  protected final static int PRECISE_RIGHT_RULER = 115;
  protected final static int DRAG_SEGMENT        = 116;
  protected int Dragging = BiSliderPresentation.NONE;
  protected int LeftValue = 0;
  protected int RightValue = 0;
  protected int DeplBef = 0;
  protected int DeplAft = 0;
  protected double Center = 0;

  // The width and height can change when the tool is resized
  protected int RulerWidth = 0;
  protected final static int MINIMUM_RULER_HEIGHT = 20;
  protected final static int PREFERRED_RULER_HEIGHT = 20;
  protected int RulerHeight = BiSliderPresentation.PREFERRED_RULER_HEIGHT;

  // but not the margins.
  protected final static int MARGIN_RULER_LEFT = 10;
  protected int              marginRulerTop = 13;
  protected final static int MARGIN_RULER_RIGHT = 10;
  protected final static int MARGIN_RULER_BOTTOM = 3;

  // protected Dimension PreferredSize = new Dimension(5, 5);
  // protected Dimension MinimumSize = new Dimension(5, 5);
  protected JTextField JTextFieldMin = new JTextField("");
  protected JTextField JTextFieldMax = new JTextField("");
  protected Vector<Point> LastFiveEvents = null;
  protected JPanel JPanel1 = new JPanel();
  protected JSlider JSlider1 = new JSlider(SwingConstants.VERTICAL, -100, 100, 0);

  protected JLabel JLabel1 =  new JLabel(){
      @Override
      public void paint(Graphics g){
        if (Ctrl.UnitPainterSupport1.getPainterListenerNumber()>0){
        try{
          g.setColor(JLabel1.getBackground());
          g.fillRect(0, 0, getWidth(), getHeight());
          g.setColor(JLabel1.getForeground());
          double Val = Double.parseDouble(JLabel1.getText());
          Rectangle bounds = new Rectangle(0, 0, getWidth(), getHeight());
          Ctrl.UnitPainterSupport1.firePaintTick(JLabel1, (Graphics2D)g, Val, -1, Ctrl.getForeground(), bounds);
        } catch (java.lang.NumberFormatException ex){
          ex.printStackTrace();
        }
        } else
          super.paint(g);
      }
    };
  protected ContentPainterSupport ContentPainterSupport1 = null;

  /**
   * Contructor, create the polygons and other nested object then register
   * mouse callbacks.
   * 
   * @param Ctrl_Arg
   *            Description of the Parameter
   */
  public BiSliderPresentation(BiSlider Ctrl_Arg, ContentPainterSupport ContentPainterSupport_Arg) {
    this.Ctrl = Ctrl_Arg;
    this.ContentPainterSupport1 = ContentPainterSupport_Arg;

    this.Ctrl.setBackground(null);

    this.JPanel1.setLayout(new BorderLayout());
    this.JPanel1.setBorder(BorderFactory.createLineBorder(this.Ctrl.getForeground()));
    this.JSlider1.setPaintLabels(true);
    this.JSlider1.setMajorTickSpacing(20);
    this.JSlider1.setMinorTickSpacing(1);
    this.JPanel1.add(this.JSlider1, BorderLayout.CENTER);
    this.JPanel1.add(this.JLabel1, BorderLayout.SOUTH);
    this.JLabel1.setHorizontalAlignment(SwingConstants.CENTER);
    this.JLabel1.setOpaque(true);
    this.JSlider1.setOpaque(true);
    this.JPanel1.revalidate();

    /*
     * JSlider1.addMouseListener(new MouseAdapter(){ public void
     * mousePressed(MouseEvent MouseEvent_Arg){
     * System.out.println(""+MouseEvent_Arg); } });
     */

    this.JSlider1.addChangeListener(new ChangeListener() {

      public void stateChanged(ChangeEvent ChangeEvent_Arg) {

        double MiddleVal = BiSliderPresentation.this.PreciseOpenedValue;

        double Amplitude = BiSliderPresentation.this.Ctrl.getSegmentSize() * Ctrl.getPrecisionAmplitudeRatio();
        if (BiSliderPresentation.this.Dragging == BiSliderPresentation.PRECISE_RIGHT_RULER && MiddleVal - Amplitude < BiSliderPresentation.this.Ctrl.getMinimumColoredValue()) {
          MiddleVal = BiSliderPresentation.this.Ctrl.getMinimumColoredValue() + Amplitude;
        } else if (BiSliderPresentation.this.Dragging == BiSliderPresentation.PRECISE_LEFT_RULER && MiddleVal + Amplitude > BiSliderPresentation.this.Ctrl.getMaximumColoredValue()) {
          MiddleVal = BiSliderPresentation.this.Ctrl.getMaximumColoredValue() - Amplitude;
        } else if (MiddleVal - Amplitude < BiSliderPresentation.this.Ctrl.getMinimumValue()) {
          MiddleVal = BiSliderPresentation.this.Ctrl.getMinimumValue() + Amplitude;
        } else if (MiddleVal + Amplitude > BiSliderPresentation.this.Ctrl.getMaximumValue()) {
          MiddleVal = BiSliderPresentation.this.Ctrl.getMaximumValue() - Amplitude;
        }

        double Val = MiddleVal + (BiSliderPresentation.this.JSlider1.getValue()) / 100d * Amplitude;
        BiSliderPresentation.this.JLabel1.setText(BiSliderPresentation.this.Ctrl.getDecimalFormater().format(Val));

        if (BiSliderPresentation.this.Dragging == BiSliderPresentation.PRECISE_LEFT_RULER) {
          BiSliderPresentation.this.Ctrl.setMinimumColoredValue(Val);
        }
        if (BiSliderPresentation.this.Dragging == BiSliderPresentation.PRECISE_RIGHT_RULER) {
          BiSliderPresentation.this.Ctrl.setMaximumColoredValue(Val);
        }
      }
    });

    if (UIManager.getLookAndFeel().getName().equals("Metal")  || UIManager.getLookAndFeel().getName().equals("Mac OS X")  || UIManager.getLookAndFeel().getName().equals("Windows")) {
      int[] xp1 = {this.LeftValue, this.LeftValue - 7,
        this.LeftValue - 7, this.LeftValue - 6, this.LeftValue + 6,
        this.LeftValue + 7, this.LeftValue + 7
      };
      int[] yp1 = {this.marginRulerTop + this.RulerHeight + 1 - 14,
        this.marginRulerTop + this.RulerHeight + 1 - 7,
        this.marginRulerTop + this.RulerHeight + 1,
        this.marginRulerTop + this.RulerHeight + 1 + 1,
        this.marginRulerTop + this.RulerHeight + 1 + 1,
        this.marginRulerTop + this.RulerHeight + 1,
        this.marginRulerTop + this.RulerHeight + 1 - 7
      };
      this.TheLeft_Polygon = new Polygon(xp1, yp1, 7);

      int[] xp2 = {this.RightValue - 7, this.RightValue - 7,
        this.RightValue - 6, this.RightValue + 6,
        this.RightValue + 7, this.RightValue + 7, this.RightValue
      };
      int[] yp2 = {this.marginRulerTop - 1 + 7,
        this.marginRulerTop - 1, this.marginRulerTop - 1 - 1,
        this.marginRulerTop - 1 - 1, this.marginRulerTop - 1,
        this.marginRulerTop - 1 + 7,
        this.marginRulerTop - 1 + 14
      ,
           };
      this.TheRight_Polygon = new Polygon(xp2, yp2, 7);

    } else { // Triangles by default
      this.TheLeft_Polygon = new Polygon();
      this.TheLeft_Polygon.addPoint(this.LeftValue, this.marginRulerTop + this.RulerHeight - this.RulerHeight / 2 - 1);
      this.TheLeft_Polygon.addPoint(this.LeftValue + this.RulerHeight / 2 + 2, this.marginRulerTop + this.RulerHeight + 1);
      this.TheLeft_Polygon.addPoint(this.LeftValue, this.marginRulerTop + this.RulerHeight + 1);
      this.TheLeft_Polygon.addPoint(this.LeftValue, this.marginRulerTop + this.RulerHeight - this.RulerHeight / 2 - 1);

      this.TheRight_Polygon = new Polygon();
      this.TheRight_Polygon.addPoint(this.RightValue - 2 - this.RulerHeight / 2, this.marginRulerTop - 1);
      this.TheRight_Polygon.addPoint(this.RightValue,
              this.marginRulerTop - 1);
      this.TheRight_Polygon.addPoint(this.RightValue,
              this.marginRulerTop + this.RulerHeight / 2 + 1);
      this.TheRight_Polygon.addPoint(this.RightValue - 2 - this.RulerHeight / 2, this.marginRulerTop - 1);
    }
    this.JComponent1 = this.Ctrl;

    this.JComponent1.addMouseListener(this);
    this.JComponent1.addMouseMotionListener(this);
    this.JComponent1.addMouseWheelListener(this);
    this.JComponent1.addComponentListener(this);

    this.JComponent1.setLayout(null);
    // PreferredSize =
    // MinimumSize = );
    this.Ctrl.setPreferredSize(new Dimension(
            BiSliderPresentation.MARGIN_RULER_LEFT + BiSliderPresentation.MARGIN_RULER_RIGHT + 16 * this.Ctrl.getSegmentCount(), this.marginRulerTop + BiSliderPresentation.MARGIN_RULER_BOTTOM + BiSliderPresentation.MINIMUM_RULER_HEIGHT));
    this.Ctrl.setMinimumSize(new Dimension(
            BiSliderPresentation.MARGIN_RULER_LEFT + BiSliderPresentation.MARGIN_RULER_RIGHT + 8 * this.Ctrl.getSegmentCount(), this.marginRulerTop + BiSliderPresentation.MARGIN_RULER_BOTTOM + BiSliderPresentation.PREFERRED_RULER_HEIGHT));

    this.JTextFieldMin.setVisible(false);
    this.JTextFieldMax.setVisible(false);

    this.JTextFieldMin.getDocument().addDocumentListener(
            new DocumentListener() {

              public void changedUpdate(DocumentEvent DocumentEvent_Arg) {
              }

              public void insertUpdate(DocumentEvent DocumentEvent_Arg) {
                String Text = BiSliderPresentation.this.JTextFieldMin.getText();
                if (!BiSliderPresentation.this.Ctrl.getUnit().equals(
                        "alpha")) {
                  try {
                    double Val = Double.parseDouble(Text);
                    BiSliderPresentation.this.JTextFieldMin.setBackground(Color.WHITE);
                  } catch (NumberFormatException NumberFormatException_Arg) {
                    BiSliderPresentation.this.JTextFieldMin.setBackground(new Color(255, 128, 128));
                  }
                }
              }

              public void removeUpdate(DocumentEvent DocumentEvent_Arg) {
                String Text = BiSliderPresentation.this.JTextFieldMin.getText();
                if (!BiSliderPresentation.this.Ctrl.getUnit().equals(
                        "alpha")) {
                  try {
                    double Val = Double.parseDouble(Text);
                    BiSliderPresentation.this.JTextFieldMin.setBackground(Color.WHITE);
                  } catch (NumberFormatException NumberFormatException_Arg) {
                    BiSliderPresentation.this.JTextFieldMin.setBackground(new Color(255, 128, 128));
                  }
                }
              }
            });

    this.JTextFieldMin.addKeyListener(new KeyAdapter() {

      @Override
      public void keyTyped(KeyEvent e) {
        if (e.getKeyChar() == KeyEvent.VK_ESCAPE) {
          while (BiSliderPresentation.this.JTextFieldMin.getActionListeners().length > 0) {
            BiSliderPresentation.this.JTextFieldMin.removeActionListener(BiSliderPresentation.this.JTextFieldMin.getActionListeners()[0]);
          }
          BiSliderPresentation.this.JTextFieldMin.setVisible(false);
        }

        BiSliderPresentation.this.JTextFieldMin.setSize(
                BiSliderPresentation.this.JTextFieldMin.getPreferredSize().width + 10,
                BiSliderPresentation.this.JTextFieldMin.getPreferredSize().height);
      }
    });

    this.JComponent1.add(this.JTextFieldMin);

    this.JTextFieldMax.getDocument().addDocumentListener(
            new DocumentListener() {

              public void changedUpdate(DocumentEvent DocumentEvent_Arg) {
              }

              public void insertUpdate(DocumentEvent DocumentEvent_Arg) {
                String Text = BiSliderPresentation.this.JTextFieldMax.getText();
                if (!BiSliderPresentation.this.Ctrl.getUnit().equals(
                        "alpha")) {
                  try {
                    double Val = Double.parseDouble(Text);
                    BiSliderPresentation.this.JTextFieldMax.setBackground(Color.WHITE);
                  } catch (NumberFormatException NumberFormatException_Arg) {
                    BiSliderPresentation.this.JTextFieldMax.setBackground(new Color(255, 128, 128));
                  }
                }
              }

              public void removeUpdate(DocumentEvent DocumentEvent_Arg) {
                String Text = BiSliderPresentation.this.JTextFieldMax.getText();
                if (!BiSliderPresentation.this.Ctrl.getUnit().equals(
                        "alpha")) {
                  try {
                    double Val = Double.parseDouble(Text);
                    BiSliderPresentation.this.JTextFieldMax.setBackground(Color.WHITE);
                  } catch (NumberFormatException NumberFormatException_Arg) {
                    BiSliderPresentation.this.JTextFieldMax.setBackground(new Color(255, 128, 128));
                  }
                }
              }
            });

    this.JTextFieldMax.addKeyListener(new KeyAdapter() {

      @Override
      public void keyTyped(KeyEvent e) {
        if (e.getKeyChar() == KeyEvent.VK_ESCAPE) {
          while (BiSliderPresentation.this.JTextFieldMax.getActionListeners().length > 0) {
            BiSliderPresentation.this.JTextFieldMax.removeActionListener(BiSliderPresentation.this.JTextFieldMax.getActionListeners()[0]);
          }
          BiSliderPresentation.this.JTextFieldMax.setVisible(false);
        }

        int OldX = BiSliderPresentation.this.JTextFieldMax.getLocation().x;
        int OldWidth = BiSliderPresentation.this.JTextFieldMax.getSize().width;
        BiSliderPresentation.this.JTextFieldMax.setSize(
                BiSliderPresentation.this.JTextFieldMax.getPreferredSize().width + 10,
                BiSliderPresentation.this.JTextFieldMax.getPreferredSize().height);
        BiSliderPresentation.this.JTextFieldMax.setLocation(
                OldX + (OldWidth - BiSliderPresentation.this.JTextFieldMax.getSize().width),
                BiSliderPresentation.this.JTextFieldMax.getLocation().y);
      }
    });

    this.JComponent1.add(this.JTextFieldMax);

    this.JComponent1.setDoubleBuffered(true);
  // JComponent1.setSize(300, 50);
  } // constructor()

  /**
   * @param Color_Arg
   *            Description of the Parameter
   * @return the opposite color to make shadow of text with it
   */
  protected Color getOppositeColor(Color Color_Arg) {
    int R = Color_Arg.getRed();
    int G = Color_Arg.getGreen();
    int B = Color_Arg.getBlue();
    R = 255 - R;
    int R2 = (Color_Arg.getRed() + 128) % 255;
    if (Math.abs(R - Color_Arg.getRed()) < Math.abs(R2 - Color_Arg.getRed())) {
      R = R2;
    }

    G = 255 - G;
    int G2 = (Color_Arg.getGreen() + 128) % 255;
    if (Math.abs(G - Color_Arg.getGreen()) < Math.abs(G2 - Color_Arg.getGreen())) {
      G = G2;
    }

    B = 255 - B;
    int B2 = (Color_Arg.getBlue() + 128) % 255;
    if (Math.abs(B - Color_Arg.getBlue()) < Math.abs(B2 - Color_Arg.getBlue())) {
      B = B2;
    }

    return new Color(R, G, B);
  } // getOppositeColor()

  /**
   * Sets the rulers value if the new values are coherents.
   * 
   * @param Min_Arg
   *            the value of the min triangle
   * @param Max_Arg
   *            the value of the max triangle
   */
  public void setRulerValues(double Min_Arg, double Max_Arg) {

    int SegmentCount = this.Ctrl.getSegmentCount();

    if (SegmentCount == 0) {
      SegmentCount = 1;
    }

    this.RulerWidth = this.JComponent1.getSize().width - BiSliderPresentation.MARGIN_RULER_LEFT - BiSliderPresentation.MARGIN_RULER_RIGHT;
    this.GraduationWidth = (this.RulerWidth * this.Ctrl.getSegmentSize()) / (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue());

    this.NeverDrawn = false;

    int NewLeftValue = (int) (BiSliderPresentation.MARGIN_RULER_LEFT + ((Min_Arg - this.Ctrl.getMinimumValue()) * this.RulerWidth) / (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()));
    int NewRightValue = (int) (BiSliderPresentation.MARGIN_RULER_LEFT + ((Max_Arg - this.Ctrl.getMinimumValue()) * this.RulerWidth) / (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()));

    if (NewLeftValue <= NewRightValue && NewLeftValue >= BiSliderPresentation.MARGIN_RULER_LEFT && NewRightValue <= BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
      this.LeftValue = NewLeftValue;
      this.RightValue = NewRightValue;

      int TriangleSide = this.RulerHeight / 2;
      TriangleSide = Math.max(Math.min(TriangleSide, 20), 10);

      if (UIManager.getLookAndFeel().getName().equals("Metal") || UIManager.getLookAndFeel().getName().equals("Mac OS X") || UIManager.getLookAndFeel().getName().equals("Windows")) {
        int[] xp1 = {this.LeftValue, this.LeftValue - 7,
          this.LeftValue - 7, this.LeftValue - 6,
          this.LeftValue + 6, this.LeftValue + 7,
          this.LeftValue + 7
        };
        int[] yp1 = {
          this.marginRulerTop + this.RulerHeight + 1 - 14,
          this.marginRulerTop + this.RulerHeight + 1 - 7,
          this.marginRulerTop + this.RulerHeight + 1,
          this.marginRulerTop + this.RulerHeight + 1 + 1,
          this.marginRulerTop + this.RulerHeight + 1 + 1,
          this.marginRulerTop + this.RulerHeight + 1,
          this.marginRulerTop + this.RulerHeight + 1 - 7
        };
        this.TheLeft_Polygon = new Polygon(xp1, yp1, 7);

        int[] xp2 = {this.RightValue - 7, this.RightValue - 7,
          this.RightValue - 6, this.RightValue + 6,
          this.RightValue + 7, this.RightValue + 7,
          this.RightValue
        };
        int[] yp2 = {this.marginRulerTop - 1 + 7,
          this.marginRulerTop - 1,
          this.marginRulerTop - 1 - 1,
          this.marginRulerTop - 1 - 1,
          this.marginRulerTop - 1,
          this.marginRulerTop - 1 + 7,
          this.marginRulerTop - 1 + 14
        ,
             };
        this.TheRight_Polygon = new Polygon(xp2, yp2, 7);

      } else { // Triangles by default
        this.TheLeft_Polygon = new Polygon();
        this.TheLeft_Polygon.addPoint(this.LeftValue,
                this.marginRulerTop + this.RulerHeight - TriangleSide - 1);
        this.TheLeft_Polygon.addPoint(
                this.LeftValue + TriangleSide + 2,
                this.marginRulerTop + this.RulerHeight + 1);
        this.TheLeft_Polygon.addPoint(this.LeftValue,
                this.marginRulerTop + this.RulerHeight + 1);
        this.TheLeft_Polygon.addPoint(this.LeftValue,
                this.marginRulerTop + this.RulerHeight - TriangleSide - 1);

        this.TheRight_Polygon = new Polygon();
        this.TheRight_Polygon.addPoint(this.RightValue - 2 - TriangleSide, this.marginRulerTop - 1);
        this.TheRight_Polygon.addPoint(this.RightValue,
                this.marginRulerTop - 1);
        this.TheRight_Polygon.addPoint(this.RightValue,
                this.marginRulerTop + TriangleSide + 1);
        this.TheRight_Polygon.addPoint(this.RightValue - 2 - TriangleSide, this.marginRulerTop - 1);
      }
    } else if (this.Ctrl.getSize().width == 0 || this.Ctrl.getSize().height == 0) {

    } else {
      System.err.println("\nsetRulerValues()");
      System.err.println("  Size              = " + this.Ctrl.getSize());
      System.err.println("  NewLeftValue      = " + NewLeftValue);
      System.err.println("  NewRightValue     = " + NewRightValue);
      System.err.println("  MARGIN_RULER_LEFT = " + BiSliderPresentation.MARGIN_RULER_LEFT);
      System.err.println("  MARGIN_RULER_LEFT + RulerWidth = " + (BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth));
    // Debug.debug(0, "");
    }
  } // setRulerValues()


  /**
   * Method called by the awt-swing mechanism when the area needs to be
   * refreshed
   * 
   * @param graphics_Arg
   *            the graphic context to draw things
   */
  @Override
  public void update(Graphics graphics_Arg, JComponent comp) {
    Color BackColor = null;
    Object SliderBackColor = UIManager.getLookAndFeel().getDefaults().get(
            "Slider.background");
    if (SliderBackColor != null || !(SliderBackColor instanceof ColorUIResource)) {
      BackColor = (Color) SliderBackColor;
    }
    if (BackColor == null || this.Ctrl.getBackground() != null) {
      BackColor = this.Ctrl.getBackground();
    }

    if (comp.isOpaque() && BackColor != null) {
      graphics_Arg.setColor(BackColor);
      graphics_Arg.fillRect(0, 0, this.JComponent1.getSize().width, this.JComponent1.getSize().height);
    }
    this.paint(graphics_Arg);
  }// update()


  /**
   * Method called by update()
   * 
   * @param graphics_Arg
   *            the graphic context to draw things
   */
  public void paint(Graphics graphics_Arg) {

    if (graphics_Arg == null) {
      return;
    }

    Graphics2D Graphics2 = (Graphics2D) graphics_Arg;
    RenderingHints RenderingHints2 = new RenderingHints(null);
    RenderingHints2.put(RenderingHints.KEY_INTERPOLATION,
            RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
    RenderingHints2.put(RenderingHints.KEY_TEXT_ANTIALIASING,
            RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    RenderingHints2.put(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_OFF);
    RenderingHints2.put(RenderingHints.KEY_RENDERING,
            RenderingHints.VALUE_RENDER_SPEED);
    Graphics2.setRenderingHints(RenderingHints2);

    Font Font1 = this.Ctrl.getFont();
    Font Font2 = new Font(Font1.getName(), Font.BOLD, Font1.getSize());
    Shape OldClip = Graphics2.getClip();
    // Shape NewClip = new Rectangle2D.Float(MARGIN_RULER_LEFT,
    // marginRulerTop,RulerWidth, RulerHeight);
    Shape NewClip = new RoundRectangle2D.Float(
            BiSliderPresentation.MARGIN_RULER_LEFT, this.marginRulerTop,
            this.RulerWidth, this.RulerHeight, this.Ctrl.getArcSize(),
            this.Ctrl.getArcSize());
    // Graphics2.setColor(Color.RED);
    // Graphics2.fill(NewClip);

    int SegmentCount = this.Ctrl.getSegmentCount();
    FontMetrics fontMetrics0 = Graphics2.getFontMetrics();
    this.RulerWidth = this.JComponent1.getSize().width - BiSliderPresentation.MARGIN_RULER_LEFT - BiSliderPresentation.MARGIN_RULER_RIGHT;
    double ValuesWidth = this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue();

    if (this.NeverDrawn) {
      this.LeftValue = (int) (BiSliderPresentation.MARGIN_RULER_LEFT + ((this.Ctrl.getMinimumColoredValue() - this.Ctrl.getMinimumValue()) * this.RulerWidth) / (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()));
      this.RightValue = (int) (BiSliderPresentation.MARGIN_RULER_LEFT + ((this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumValue()) * this.RulerWidth) / (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()));
      this.TheRight_Polygon.translate(this.RulerWidth, 0);
      this.NeverDrawn = false;
    }

    if (this.RulerWidth != this.JComponent1.getSize().width - BiSliderPresentation.MARGIN_RULER_LEFT - BiSliderPresentation.MARGIN_RULER_RIGHT) {
      int NewRulerWidth = this.JComponent1.getSize().width - BiSliderPresentation.MARGIN_RULER_LEFT - BiSliderPresentation.MARGIN_RULER_RIGHT;

      NewRulerWidth = NewRulerWidth - NewRulerWidth % SegmentCount;

      int NewLeftValue = BiSliderPresentation.MARGIN_RULER_LEFT + ((this.LeftValue - BiSliderPresentation.MARGIN_RULER_LEFT) * (NewRulerWidth)) / this.RulerWidth;
      int NewRightValue = BiSliderPresentation.MARGIN_RULER_LEFT + ((this.RightValue - BiSliderPresentation.MARGIN_RULER_LEFT) * (NewRulerWidth)) / this.RulerWidth;

      this.TheLeft_Polygon.translate(NewLeftValue - this.LeftValue, 0);
      this.TheRight_Polygon.translate(NewRightValue - this.RightValue, 0);

      this.LeftValue = NewLeftValue;
      this.RightValue = NewRightValue;

      this.RulerWidth = NewRulerWidth;

      this.GraduationWidth = (float) ((this.RulerWidth * this.Ctrl.getSegmentSize()) / (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()));
    }

    Graphics2.setClip(NewClip);

    if (!UIManager.getLookAndFeel().getName().equals("Metal") && !UIManager.getLookAndFeel().getName().equals("Mac OS X") && !UIManager.getLookAndFeel().getName().equals("Windows")) {
      System.out.println("sory for you the ui is "+UIManager.getLookAndFeel().getName());
      Graphics2.setColor(this.Ctrl.getSliderBackground());
      Graphics2.fillRoundRect(BiSliderPresentation.MARGIN_RULER_LEFT,
              this.marginRulerTop, this.RulerWidth, this.RulerHeight,
              this.Ctrl.getArcSize(), this.Ctrl.getArcSize());
    } else {
      Object SliderBackColor = UIManager.getLookAndFeel().getDefaults().get("Slider.gradient");
      if (SliderBackColor != null && SliderBackColor instanceof List) {
        List GradientProperties = (List) SliderBackColor;
        Float Cut1 = (Float) GradientProperties.get(0);
        Float Cut2 = (Float) GradientProperties.get(1);
        Color Color1 = (Color) GradientProperties.get(2);
        Color Color2 = (Color) GradientProperties.get(3);
        Color Color3 = (Color) GradientProperties.get(4);
        float Rem = 1f - Cut2 - 2 * Cut1;
        int x1 = BiSliderPresentation.MARGIN_RULER_LEFT;
        int x2 = this.RulerWidth + BiSliderPresentation.MARGIN_RULER_LEFT;
        int y1 = this.marginRulerTop;
        int y2 = this.marginRulerTop + this.RulerHeight;

        GradientPaint GradientPaint1 = new GradientPaint(x1, y1,
                Color3, x1, y1 + (int) (this.RulerHeight * Rem), Color2);
        GradientPaint GradientPaint2 = new GradientPaint(x1, y1 + (int) (this.RulerHeight * Rem), Color2, x1, y1 + (int) (this.RulerHeight * (Rem + Cut1)), Color1);
        GradientPaint GradientPaint3 = new GradientPaint(x1, y1 + (int) (this.RulerHeight * (Rem + Cut1 + Cut2)),
                Color1, x1, y2, Color.WHITE);

        Graphics2.setPaint(GradientPaint1);
        Graphics2.fillRect(x1, y1, this.RulerWidth,
                (int) (this.RulerHeight * Rem));
        Graphics2.setPaint(GradientPaint2);
        Graphics2.fillRect(x1, y1 + (int) (this.RulerHeight * Rem),
                this.RulerWidth, (int) (this.RulerHeight * Cut1) + 1);
        Graphics2.setColor(Color1);
        Graphics2.fillRect(x1, y1 + (int) (this.RulerHeight * (Rem + Cut1)),
                this.RulerWidth, (int) (this.RulerHeight * Cut2) + 1);
        Graphics2.setPaint(GradientPaint3);
        Graphics2.fillRect(x1, y1 + (int) (this.RulerHeight * (Rem + Cut1 + Cut2)),
                this.RulerWidth, (int) (this.RulerHeight * Cut1) + 1);
      }
    }

    // Graphics2.fillRoundRect(MARGIN_RULER_LEFT, marginRulerTop,
    // RulerWidth, RulerHeight, Ctrl.getArcSize(), Ctrl.getArcSize());
    Graphics2.setColor(this.Ctrl.getSliderBackground().darker());
    Graphics2.drawArc(BiSliderPresentation.MARGIN_RULER_LEFT + 1,
            this.marginRulerTop, this.Ctrl.getArcSize(), this.Ctrl.getArcSize(), 90, 90);
    Graphics2.drawArc(BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth - this.Ctrl.getArcSize() - 1,
            this.marginRulerTop, this.Ctrl.getArcSize(), this.Ctrl.getArcSize(), 0, 90);

    Graphics2.setColor(this.Ctrl.getSliderBackground().brighter());
    Graphics2.drawArc(BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth - this.Ctrl.getArcSize() - 1,
            this.marginRulerTop + this.RulerHeight - this.Ctrl.getArcSize(), this.Ctrl.getArcSize(),
            this.Ctrl.getArcSize(), 270, 90);
    Graphics2.drawArc(BiSliderPresentation.MARGIN_RULER_LEFT + 1,
            this.marginRulerTop + this.RulerHeight - this.Ctrl.getArcSize(), this.Ctrl.getArcSize(),
            this.Ctrl.getArcSize(), 180, 90);

    Graphics2.setColor(this.Ctrl.getSliderBackground().darker());
    Graphics2.drawLine(BiSliderPresentation.MARGIN_RULER_LEFT + this.Ctrl.getArcSize() / 2 - 1, this.marginRulerTop + 1,
            BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth - this.Ctrl.getArcSize() / 2 + 1,
            this.marginRulerTop + 1);
    Graphics2.drawLine(BiSliderPresentation.MARGIN_RULER_LEFT + 1,
            this.marginRulerTop + this.Ctrl.getArcSize() / 2,
            BiSliderPresentation.MARGIN_RULER_LEFT + 1,
            this.marginRulerTop + this.RulerHeight - this.Ctrl.getArcSize() / 2);
    Graphics2.setColor(this.Ctrl.getSliderBackground().brighter());
    Graphics2.drawLine(BiSliderPresentation.MARGIN_RULER_LEFT + this.Ctrl.getArcSize() / 2 - 1, this.marginRulerTop + this.RulerHeight - 1, BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth - this.Ctrl.getArcSize() / 2 + 1,
            this.marginRulerTop + this.RulerHeight - 1);
    Graphics2.drawLine(BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth - 1, this.marginRulerTop + this.Ctrl.getArcSize() / 2 + 1,
            BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth - 1,
            this.marginRulerTop + this.RulerHeight - this.Ctrl.getArcSize() / 2);
    Graphics2.setClip(OldClip);

    int LastMax = 0;

    /*
     * NewClip = new RoundRectangle2D.Float( MARGIN_RULER_LEFT+2,
     * marginRulerTop, RulerWidth-3, RulerHeight, Ctrl.getArcSize(),
     * Ctrl.getArcSize());
     */

    // it is the right opportunity to look for how many full non colored
    // segments there are before the fisrt one
    int SegmentCountBefore = 0;
    double[][] ColTable = this.Ctrl.getColorTable();
    double mincv = Ctrl.abstr.MinimumColoredValue;
    double maxcv = Ctrl.abstr.MaximumColoredValue;
    Ctrl.abstr.MinimumColoredValue = Ctrl.abstr.MinimumValue;
    Ctrl.abstr.MaximumColoredValue = Ctrl.abstr.MaximumValue;
    double[][] ColTable2 = this.Ctrl.getColorTable();
    Ctrl.abstr.MinimumColoredValue = mincv;
    Ctrl.abstr.MaximumColoredValue = maxcv;

    String Unit = this.Ctrl.getUnit();

    // Graphics2.setClip(NewClip);
    for (int i = 0; i <= SegmentCount; i++) {
      Color color2 = null;
      if (i<SegmentCount)
        color2 = new Color((int)ColTable2[i][2]);
      else
        color2 = Ctrl.getMaximumColor();
      double Val = this.Ctrl.getMinimumValue() + i * this.Ctrl.getSegmentSize();
      double Val2 = this.Ctrl.getMinimumValue() + (i + 1) * this.Ctrl.getSegmentSize();

      if (Val <= ColTable[0][0] && ColTable[0][0] < Val2) {
        SegmentCountBefore = i;
      }

      String NumberString = "";
      int NumberWidth = 0;
      int x = 0;

      String MaxNumberString = "";
      if (Unit.equals("alpha")) {
        MaxNumberString = "" + (char) ((int) this.Ctrl.getMaximumValue() + 'A');
      } else if (this.Ctrl.getMaximumValue() == (long) this.Ctrl.getMaximumValue()) {
        MaxNumberString = "" + ((long) this.Ctrl.getMaximumValue()) + Unit;
      } else {
        MaxNumberString = "" + (((long) (this.Ctrl.getMaximumValue() * 10)) / 10) + Unit;
      }

      if (Val > this.Ctrl.getMaximumValue()) {
        Val = this.Ctrl.getMaximumValue();
        if (Unit.equals("alpha")) {
          NumberString = "" + (char) ((int) Val + 'A');
        } else if (Val == (long) Val) {
          NumberString = "" + ((long) Val) + Unit;
        } else {
          NumberString = "" + (((long) (Val * 10)) / 10f) + Unit;
        }
        NumberWidth = fontMetrics0.stringWidth(NumberString);

        x = this.JComponent1.getSize().width - BiSliderPresentation.MARGIN_RULER_RIGHT - NumberWidth;
      } else {
        if (Unit.equals("alpha")) {
          NumberString = "" + (char) ((int) Val + 'A');
        } else if (Val == (long) Val) {
          NumberString = "" + ((long) Val) + Unit;
        } else {
          NumberString = "" + (((long) (Val * 10)) / 10f) + Unit;
        }
        NumberWidth = fontMetrics0.stringWidth(NumberString);

        x = BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * i) - (int) ((NumberWidth * ((float) i / SegmentCount)));
      }

      // get the color
      graphics_Arg.setColor(this.Ctrl.getForeground());


      if (Val == this.Ctrl.getMaximumValue() || Val == this.Ctrl.getMinimumValue())
        Graphics2.setFont(Font2);

      if (Ctrl.UnitPainterSupport1.getPainterListenerNumber()>0) {
        int x0 = BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * (i-1));
        int x3 = BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * (i+1));
        Rectangle rect0 = new Rectangle(x0,  1, x3 - x0, this.marginRulerTop-1);
        Ctrl.UnitPainterSupport1.firePaintTick(this.Ctrl, Graphics2, Val, i, color2, rect0);
      } else if (x + NumberWidth < this.JComponent1.getSize().width - BiSliderPresentation.MARGIN_RULER_RIGHT - fontMetrics0.stringWidth(MaxNumberString) && x >= LastMax + 2)
        Graphics2.drawString(NumberString, x, this.marginRulerTop - 1);


      // restore the font
      Graphics2.setFont(Font1);
      LastMax = x + NumberWidth;

      if (LastMax == x + NumberWidth) {
        Graphics2.setColor(this.Ctrl.getSliderBackground().darker().darker());
      } else {
        Graphics2.setColor(this.Ctrl.getSliderBackground().darker());
      }

      if (this.ContentPainterSupport1.getPainterListenerNumber() > 0 && Val < this.Ctrl.getMaximumValue()) {
        int x0 = BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * i);
        int x3 = BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * (i + 1));
        Rectangle Rect1bis = new Rectangle(x0,
                this.marginRulerTop + 2, x3 - x0,
                this.RulerHeight - 3);

        x3 = Math.min(x3, BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth);
        Rectangle Rect1 = new Rectangle(x0, this.marginRulerTop + 2,
                x3 - x0, this.RulerHeight - 3);

        Shape ShapeClip = Graphics2.getClip();
        Color Color0 = Graphics2.getColor();
        Graphics2.setClip(NewClip);
        Graphics2.clip(Rect1);

        this.ContentPainterSupport1.firePaint(this.Ctrl, Graphics2,
                Val, Val2, i, null, color2, Rect1bis, Rect1bis);
        Graphics2.setClip(ShapeClip);
        Graphics2.setColor(Color0);
      }

      if (i != 0 && Val < this.Ctrl.getMaximumValue()) {
        if (this.ContentPainterSupport1.getPainterListenerNumber() == 0) {
          Graphics2.drawLine(BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * i),
                  this.marginRulerTop + 2,
                  BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * i),
                  this.marginRulerTop + this.RulerHeight - 2);
        }
      }

      if (i == 0) {
        this.RectFirstLabel = new Rectangle(x - 1,
                this.marginRulerTop - fontMetrics0.getAscent(),
                NumberWidth + 1, fontMetrics0.getAscent());
      }

      if (i == SegmentCount) {
        this.RectLastLabel = new Rectangle(x - 1, this.marginRulerTop - fontMetrics0.getAscent(), NumberWidth + 1,
                fontMetrics0.getAscent());
      }

      if (this.MouseUnder == this.FIRST_LABEL) {
        Graphics2.draw(this.RectFirstLabel);
      }
      if (this.MouseUnder == this.LAST_LABEL) {
        Graphics2.draw(this.RectLastLabel);
      }
    }

    if (this.MouseUnder == BiSliderPresentation.SEGMENT && this.RectangleSegment != null) {
      Graphics2.setColor(SystemColor.control.darker());
      Graphics2.draw(this.RectangleSegment);
    }

    // ColorisationEvent CE1 = new ColorisationEvent(this,
    // Ctrl.getColorTable());
    // colored segment
    for (int i = 0; i < ColTable.length; i++) {
      Graphics2.setColor(new Color((int) (ColTable[i][2])));
      if (ColTable[i][0] < ColTable[i][1]) {
        // the selection only
        int x1 = (int) (BiSliderPresentation.MARGIN_RULER_LEFT + (this.RulerWidth * (ColTable[i][0] - this.Ctrl.getMinimumValue())) / ValuesWidth) + 1;
        int x2 = (int) (BiSliderPresentation.MARGIN_RULER_LEFT + (this.RulerWidth * ((ColTable[i][1]) - this.Ctrl.getMinimumValue())) / ValuesWidth);

        // the full segment
        int x0 = BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * (int) ColTable[i][3]);
        int x3 = BiSliderPresentation.MARGIN_RULER_LEFT + (int) (this.GraduationWidth * (int) (ColTable[i][3] + 1));
        Rectangle Rect1bis = new Rectangle(x1,
                this.marginRulerTop + 2, x2 - x1,
                this.RulerHeight - 3);

        x3 = Math.min(x3, BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth);
        Rectangle Rect1 = new Rectangle(x1, this.marginRulerTop + 2,
                x2 - x1, this.RulerHeight - 3);
        Rectangle RectClip = new Rectangle(x1 - 1,
                this.marginRulerTop + 2, x2 - x1 + 1,
                this.RulerHeight - 3);
        Rectangle Rect2 = new Rectangle(x0, this.marginRulerTop + 2,
                x3 - x0, this.RulerHeight - 3);

        if (this.ContentPainterSupport1.getPainterListenerNumber() == 0) {
          Graphics2.fill(Rect1);
          if (i != 0) {
            Graphics2.setColor(new Color(Graphics2.getColor().getRed(), Graphics2.getColor().getGreen(),
                    Graphics2.getColor().getBlue(), 192));
            Graphics2.drawLine(x1 - 1, this.marginRulerTop + 3,
                    x1 - 1, this.marginRulerTop + this.RulerHeight - 3);
          }
        } else {
          Shape ShapeClip = Graphics2.getClip();
          Color Color0 = Graphics2.getColor();
          Graphics2.setClip(NewClip);
          Graphics2.clip(RectClip);
          this.ContentPainterSupport1.firePaint(this.Ctrl, Graphics2,
                  ColTable[i][0], ColTable[i][1], i + SegmentCountBefore, Graphics2.getColor(), Graphics2.getColor(),
                  Rect1bis, Rect2);
          Graphics2.setClip(OldClip);
          Graphics2.setColor(Color0);
        }
      }
    }

    Graphics2.setClip(OldClip);

    if (this.Dragging == BiSliderPresentation.ALT_LEFT_RULER || this.Dragging == BiSliderPresentation.ALT_RIGHT_RULER) {
      Rectangle Rect1 = this.TheRight_Polygon.getBounds();
      Rectangle Rect2 = this.TheLeft_Polygon.getBounds();
      Rectangle2D Rectangle2D1 = Rect1.createUnion(Rect2);

      Graphics2.setColor(SystemColor.scrollbar.darker());
      Graphics2.fillRect((int) (Rectangle2D1.getX() + Rectangle2D1.getWidth() / 2), this.marginRulerTop, 2,
              this.RulerHeight);
    }

    // Graphics2.setColor(getOppositeColor(Ctrl.getBackground()));
    Graphics2.setColor(Color.BLACK);
    Graphics2.drawRoundRect(BiSliderPresentation.MARGIN_RULER_LEFT,
            this.marginRulerTop, this.RulerWidth, this.RulerHeight,
            this.Ctrl.getArcSize(), this.Ctrl.getArcSize());

    this.paintThumbs(Graphics2);

    if (this.Dragging == BiSliderPresentation.ALT_RIGHT_RULER || this.Dragging == BiSliderPresentation.ALT_LEFT_RULER || this.Dragging == BiSliderPresentation.RIGHT_RULER || this.Dragging == BiSliderPresentation.SEGMENT || 
        this.Dragging == BiSliderPresentation.SHIFT_SEGMENT || this.Dragging == BiSliderPresentation.SHIFT_RIGHT_RULER|| this.Dragging == BiSliderPresentation.CTRL_RIGHT_RULER) {
      String ValString = "";
      if (Unit.equals("alpha")) {
        ValString = "" + (char) ((int) this.Ctrl.getMaximumColoredValue() + 'A');
      } else if (Math.abs(((int) this.Ctrl.getMaximumColoredValue()) - this.Ctrl.getMaximumColoredValue()) < 0.0001) {
        ValString = "" + (int) this.Ctrl.getMaximumColoredValue();
      } else {
        ValString = this.Ctrl.getDecimalFormater().format(
                this.Ctrl.getMaximumColoredValue());
      }

      // display selected value when dragged
      if (Ctrl.UnitPainterSupport1.getPainterListenerNumber()>0) {
        Rectangle rect0 = new Rectangle(this.RightValue + 3,  this.marginRulerTop + 1, 30, fontMetrics0.getAscent()+1);
        Ctrl.UnitPainterSupport1.firePaintValue(this.Ctrl, Graphics2, this.Ctrl.getMaximumColoredValue(), this.Ctrl.getForeground(), rect0);
      } else {
        Graphics2.setColor(this.getOppositeColor(this.Ctrl.getForeground()));
        Graphics2.drawString(ValString, this.RightValue + 3,
                this.marginRulerTop + fontMetrics0.getAscent() + 1);
        Graphics2.setColor(this.Ctrl.getForeground());
        Graphics2.drawString(ValString, this.RightValue + 2,
                this.marginRulerTop + fontMetrics0.getAscent());
      }
    }

    if (this.Dragging == BiSliderPresentation.ALT_RIGHT_RULER  || this.Dragging == BiSliderPresentation.ALT_LEFT_RULER || this.Dragging == BiSliderPresentation.LEFT_RULER ||
        this.Dragging == BiSliderPresentation.SEGMENT          || this.Dragging == BiSliderPresentation.SHIFT_SEGMENT ||
        this.Dragging == BiSliderPresentation.SHIFT_LEFT_RULER || this.Dragging == BiSliderPresentation.CTRL_LEFT_RULER) {
      String ValString = "";
      if (Unit.equals("alpha")) {
        ValString = "" + (char) ((int) this.Ctrl.getMinimumColoredValue() + 'A');
      } else if (Math.abs(((int) this.Ctrl.getMinimumColoredValue()) - this.Ctrl.getMinimumColoredValue()) < 0.0001) {
        ValString = "" + (int) this.Ctrl.getMinimumColoredValue();
      } else {
        ValString = this.Ctrl.getDecimalFormater().format(
                this.Ctrl.getMinimumColoredValue());
      }

      // display selected value when dragged
      if (Ctrl.UnitPainterSupport1.getPainterListenerNumber()>0) {
        Rectangle rect0 = new Rectangle(this.LeftValue - 33,  this.marginRulerTop + this.RulerHeight-fontMetrics0.getAscent()-1, 30, fontMetrics0.getAscent()+1);
        Ctrl.UnitPainterSupport1.firePaintValue(this.Ctrl, Graphics2, this.Ctrl.getMinimumColoredValue(), this.Ctrl.getForeground(), rect0);
      } else {
        Graphics2.setColor(this.getOppositeColor(this.Ctrl.getForeground()));
        Graphics2.drawString(ValString, this.LeftValue - fontMetrics0.stringWidth(ValString) - 1,
                this.marginRulerTop + this.RulerHeight - 1);
        Graphics2.setColor(this.Ctrl.getForeground());
        Graphics2.drawString(ValString, this.LeftValue - fontMetrics0.stringWidth(ValString) - 2,
                this.marginRulerTop + this.RulerHeight - 2);
      }
    }

    if (this.JTextFieldMax.isVisible()) {
      this.JTextFieldMax.repaint();
    }
    if (this.JTextFieldMin.isVisible()) {
      this.JTextFieldMin.repaint();
    }
  } // paint()

  /**
   * paint a thumb
   */
  public void paintThumbs(Graphics graphics_Arg) {
    Graphics2D Graphics2 = (Graphics2D) graphics_Arg;
    // System.out.println("getLookAndFeel="+UIManager.getLookAndFeel().getName());
    // System.out.println("getCurrentTheme="+MetalLookAndFeel.getCurrentTheme().getName());
    if (UIManager.getLookAndFeel().getName().equals("Metal") || UIManager.getLookAndFeel().getName().equals("Mac OS X")  || UIManager.getLookAndFeel().getName().equals("Windows")) {
      if (this.Dragging == BiSliderPresentation.LEFT_RULER || this.Dragging == BiSliderPresentation.SHIFT_LEFT_RULER || this.Dragging == BiSliderPresentation.ALT_LEFT_RULER) {
        this.paintThumb(graphics_Arg, true, this.TheLeft_Polygon,
                MetalLookAndFeel.getPrimaryControlShadow(),
                MetalLookAndFeel.getPrimaryControlDarkShadow(),
                MetalLookAndFeel.getPrimaryControl());
      } else if (this.MouseUnder == this.LEFT_POLYGON) {
        this.paintThumb(graphics_Arg, true, this.TheLeft_Polygon,
                MetalLookAndFeel.getControlShadow(), MetalLookAndFeel.getControlDarkShadow(), MetalLookAndFeel.getControl());
      } else {
        this.paintThumb(graphics_Arg, true, this.TheLeft_Polygon,
                MetalLookAndFeel.getControl(), MetalLookAndFeel.getControlShadow(), MetalLookAndFeel.getControlHighlight());
      }

      if (this.Dragging == BiSliderPresentation.RIGHT_RULER || this.Dragging == BiSliderPresentation.SHIFT_RIGHT_RULER || this.Dragging == BiSliderPresentation.ALT_RIGHT_RULER) {
        this.paintThumb(graphics_Arg, false, this.TheRight_Polygon,
                MetalLookAndFeel.getPrimaryControlShadow(),
                MetalLookAndFeel.getPrimaryControlDarkShadow(),
                MetalLookAndFeel.getPrimaryControl());
      } else if ((this.MouseUnder == this.RIGHT_POLYGON)) {
        this.paintThumb(graphics_Arg, false, this.TheRight_Polygon,
                MetalLookAndFeel.getControlShadow(), MetalLookAndFeel.getControlDarkShadow(), MetalLookAndFeel.getControl());
      } else {
        this.paintThumb(graphics_Arg, false, this.TheRight_Polygon,
                MetalLookAndFeel.getControl(), MetalLookAndFeel.getControlShadow(), MetalLookAndFeel.getControlHighlight());
      }

    } else { // default thumb = triangle
      Graphics2.setColor(SystemColor.control);
      if (this.Dragging == BiSliderPresentation.LEFT_RULER || this.Dragging == BiSliderPresentation.SHIFT_LEFT_RULER || this.Dragging == BiSliderPresentation.ALT_LEFT_RULER) {
        Graphics2.setColor(MetalLookAndFeel.getPrimaryControlShadow());
      } else if (this.MouseUnder == this.LEFT_POLYGON) {
        Graphics2.setColor(MetalLookAndFeel.getControlHighlight());
      }
      Graphics2.fillPolygon(this.TheLeft_Polygon);

      Graphics2.setColor(SystemColor.control);
      if (this.Dragging == BiSliderPresentation.RIGHT_RULER || this.Dragging == BiSliderPresentation.SHIFT_RIGHT_RULER || this.Dragging == BiSliderPresentation.ALT_RIGHT_RULER) {
        Graphics2.setColor(MetalLookAndFeel.getPrimaryControlShadow());
      } else if (this.MouseUnder == this.RIGHT_POLYGON) {
        Graphics2.setColor(MetalLookAndFeel.getControlHighlight());
      }
      Graphics2.fillPolygon(this.TheRight_Polygon);

      // shadow
      Graphics2.setColor(SystemColor.control.brighter());
      Graphics2.drawLine(this.TheLeft_Polygon.xpoints[0] + 1,
              this.TheLeft_Polygon.ypoints[0] + 2,
              this.TheLeft_Polygon.xpoints[2] + 1,
              this.TheLeft_Polygon.ypoints[2] - 1);
      Graphics2.drawLine(this.TheRight_Polygon.xpoints[0] + 1,
              this.TheRight_Polygon.ypoints[0] + 1,
              this.TheRight_Polygon.xpoints[1] - 1,
              this.TheRight_Polygon.ypoints[1] + 1);
      Graphics2.setColor(SystemColor.control.darker());
      Graphics2.drawLine(this.TheLeft_Polygon.xpoints[1] - 2,
              this.TheLeft_Polygon.ypoints[1] - 1,
              this.TheLeft_Polygon.xpoints[2] + 1,
              this.TheLeft_Polygon.ypoints[2] - 1);
      Graphics2.drawLine(this.TheRight_Polygon.xpoints[1] - 1,
              this.TheRight_Polygon.ypoints[1] + 1,
              this.TheRight_Polygon.xpoints[2] - 1,
              this.TheRight_Polygon.ypoints[2] - 1);

      if (this.MouseUnder == this.SELECTION) {
        Rectangle Rect1 = this.TheRight_Polygon.getBounds();
        Rectangle Rect2 = this.TheLeft_Polygon.getBounds();
        Rectangle2D Rectangle2D1 = Rect1.createUnion(Rect2);

        Graphics2.draw(Rectangle2D1);
      } else if (this.MouseUnder == this.LEFT_POLYGON) {
        // Graphics2.fillPolygon(TheLeft_Polygon);
        Graphics2.setColor(SystemColor.control.darker().darker());
        Graphics2.drawLine(this.TheLeft_Polygon.xpoints[0] + 1,
                this.TheLeft_Polygon.ypoints[0] + 2,
                this.TheLeft_Polygon.xpoints[2] + 1,
                this.TheLeft_Polygon.ypoints[2] - 1);
        Graphics2.setColor(SystemColor.control);
        Graphics2.drawLine(this.TheLeft_Polygon.xpoints[1] - 2,
                this.TheLeft_Polygon.ypoints[1] - 1,
                this.TheLeft_Polygon.xpoints[2] + 1,
                this.TheLeft_Polygon.ypoints[2] - 1);
      } else if (this.MouseUnder == this.RIGHT_POLYGON) {
        // Graphics2.fillPolygon(TheRight_Polygon);
        Graphics2.setColor(SystemColor.control.darker().darker());
        Graphics2.drawLine(this.TheRight_Polygon.xpoints[0] + 1,
                this.TheRight_Polygon.ypoints[0] + 1,
                this.TheRight_Polygon.xpoints[1] - 1,
                this.TheRight_Polygon.ypoints[1] + 1);
        Graphics2.setColor(SystemColor.control);
        Graphics2.drawLine(this.TheRight_Polygon.xpoints[1] - 1,
                this.TheRight_Polygon.ypoints[1] + 1,
                this.TheRight_Polygon.xpoints[2] - 1,
                this.TheRight_Polygon.ypoints[2] - 1);
      }

      // Graphics2.setColor(getOppositeColor(Ctrl.getBackground()));
      Graphics2.setColor(Color.BLACK);
      Graphics2.drawPolygon(this.TheLeft_Polygon);
      Graphics2.drawPolygon(this.TheRight_Polygon);
    }
  }// paintThumb()

  // Parameters of the widget
  final private static int PADDING_TOP_BOTTOM = 20;
  final private static int BTN_NONE = 0;
  final private static int BTN_UP = 1;
  final private static int BTN_MID = 2;
  final private static int BTN_DOWN = 3;

  /**
   * Paints shadowed dots on the given <tt>Graphics</tt> objects. The
   * coordinates of the points must be given in arrays.
   * 
   * @param g
   *            the <tt>Graphics</tt> object on which we wand to paint the
   *            dots
   * @param xCoords
   *            array containing the X coordinates of the dots
   * @param yCoords
   *            array containing the Y coordinates of the dots
   * @param num
   *            the number of dots
   * @author Christophe Jacquet, Frederic Vernier
   */
  private void paintShadowedDots(Graphics g, int[] xCoords, int[] yCoords,
          int num, Color DarkColor_Arg, Color BrightColor_Arg) {
    for (int s = 0; s < 2; s++) {
      g.setColor(s == 0 ? BrightColor_Arg : DarkColor_Arg);
      for (int i = 0; i < num; i++) {
        g.drawLine(xCoords[i] + s, yCoords[i] + s, xCoords[i] + s,
                yCoords[i] + s);
      }
    }
  }// paintShadowedDots()
  /**
   * Draws an arrow (upward or downward) on a given <tt>Graphics</tt>
   * object.
   * 
   * @param g
   *            the <tt>Graphics</tt> object on which we want to paint the
   *            arrow
   * @param up
   *            if <tt>true</tt>, the arrow will be painted upward. If
   * @author Christophe Jacquet, Frederic Vernier <tt>false</tt>, it will
   *         be painted downward
   */
  private void paintThumb(Graphics graphics_Arg, boolean up,
          Polygon Thumb_Arg, Color MainColor_Arg, Color DarkColor_Arg,
          Color BrightColor_Arg) {

    Graphics2D G2 = (Graphics2D) graphics_Arg;
    // System.out.println("UIManager.getLookAndFeel() =
    // "+UIManager.getLookAndFeel().getName());
    // System.out.println("MetalLookAndFeel.getCurrentTheme() =
    // "+MetalLookAndFeel.getCurrentTheme().getName());

    if ((UIManager.getLookAndFeel().getName().equals("Metal") && MetalLookAndFeel.getCurrentTheme().getName().equals("Ocean")) ){
      Color BackColor = null;
      Object SliderBackColor = UIManager.getLookAndFeel().getDefaults().get("Slider.gradient");
      if (SliderBackColor != null && SliderBackColor instanceof List) {
        List GradientProperties = (List) SliderBackColor;
        Float Cut1 = (Float) GradientProperties.get(0);
        Float Cut2 = (Float) GradientProperties.get(1);
        Color Color1 = (Color) GradientProperties.get(2);
        Color Color2 = (Color) GradientProperties.get(3);
        Color Color3 = (Color) GradientProperties.get(4);
        float Rem = 1f - Cut2 - 2 * Cut1;
        int x1 = up ? Thumb_Arg.xpoints[0] : Thumb_Arg.xpoints[6];
        int y1 = up ? Thumb_Arg.ypoints[0] : Thumb_Arg.ypoints[6];

        int x2 = up ? Thumb_Arg.xpoints[1] : Thumb_Arg.xpoints[5];
        int y2 = up ? Thumb_Arg.ypoints[1] : Thumb_Arg.ypoints[5];

        Shape Shape0 = G2.getClip();
        G2.setClip(Thumb_Arg);
        if (up) {
          GradientPaint GradientPaint1 = new GradientPaint(x1 - 7,
                  y1, Color3, x1, y1 + (int) (16f * Rem), Color1);
          GradientPaint GradientPaint2 = new GradientPaint(x1, y1 + (int) (16f * Rem), Color1, x1, y1 + (int) (16f * (Rem + Cut1)), Color2);
          GradientPaint GradientPaint3 = new GradientPaint(x1, y1 + (int) (16f * (Rem + Cut1 + Cut2)), Color2, x1,
                  y1 + 16, Color3);

          G2.setPaint(GradientPaint1);
          G2.fillRect(x1 - 7, y1, 15, (int) (16f * Rem));
          G2.setPaint(GradientPaint2);
          G2.fillRect(x1 - 7, y1 + (int) (16f * Rem), 15,
                  (int) (16f * Cut1) + 1);
          G2.setColor(Color2);
          G2.fillRect(x1 - 7, y1 + (int) (16f * (Rem + Cut1)), 15,
                  (int) (16f * Cut2) + 1);
          G2.setPaint(GradientPaint3);
          G2.fillRect(x1 - 7, y1 + (int) (16f * (Rem + Cut1 + Cut2)),
                  15, (int) (16f * Cut1) + 1);
        } else {
          GradientPaint GradientPaint1 = new GradientPaint(x1 - 7,
                  y1, Color3, x1, y1 - (int) (16f * Rem), Color1);
          GradientPaint GradientPaint2 = new GradientPaint(x1, y1 - (int) (16f * Rem), Color1, x1, y1 - (int) (16f * (Rem + Cut1)), Color2);
          GradientPaint GradientPaint3 = new GradientPaint(x1, y1 - (int) (16f * (Rem + Cut1 + Cut2)), Color2, x1,
                  y1 - 16, Color3);

          G2.setPaint(GradientPaint1);
          G2.fillRect(x1 - 7, y1 - (int) (16f * Rem), 15,
                  (int) (16f * Rem));
          G2.setPaint(GradientPaint2);
          G2.fillRect(x1 - 7, y1 - (int) (16f * (Rem + Cut1)), 15,
                  (int) (16f * Cut1) + 1);
          G2.setColor(Color2);
          G2.fillRect(x1 - 7, y1 - (int) (16f * (Rem + Cut1 + Cut2)),
                  15, (int) (16f * Cut2) + 1);
          G2.setPaint(GradientPaint3);
          G2.fillRect(x1 - 7, y1 - 16, 15, (int) (16f * Cut1) + 1);
        }
        G2.setClip(Shape0);
      }

    } else {
      G2.setColor(new Color(MainColor_Arg.getRed(), MainColor_Arg.getGreen(), MainColor_Arg.getBlue(), 255));
      G2.fillPolygon(Thumb_Arg);
    }
    // System.out.println( ((OceanTheme)MetalLookAndFeel.getCurrentTheme()).
    // getProperty("Button.gradient"));

    G2.setColor(Color.BLACK);
    G2.drawPolygon(Thumb_Arg);
    int yy = up ? Thumb_Arg.ypoints[0] + 8 : Thumb_Arg.ypoints[6] - 13;
    int xx = up ? Thumb_Arg.xpoints[0] : Thumb_Arg.xpoints[6];
    int[] xp = {xx - 5, xx - 1, xx + 3, xx - 3, xx + 1, xx - 5, xx - 1,
      xx + 3
    };
    int[] yp = {yy, yy, yy, yy + 2, yy + 2, yy + 4, yy + 4, yy + 4};

    this.paintShadowedDots(G2, xp, yp, 8, DarkColor_Arg, BrightColor_Arg);
  }// paintThumb()


  /**
   * Method called by the awt-swing mechanism when the user press the mouse
   * button over this component
   * 
   * @param MouseEvent_Arg
   *            the mouse event generatedby the awt-swing mechanism
   */
  public void mousePressed(MouseEvent MouseEvent_Arg) {

    this.LastFiveEvents = new Vector<Point>();
    this.LastFiveEvents.add(MouseEvent_Arg.getPoint());

    if (this.JTextFieldMin.isVisible()) {
      this.JTextFieldMin.postActionEvent();
      while (this.JTextFieldMin.getActionListeners().length > 0) {
        this.JTextFieldMin.removeActionListener(this.JTextFieldMin.getActionListeners()[0]);
      }
      this.JTextFieldMin.setVisible(false);
      this.Ctrl.requestFocus();
    }
    if (this.JTextFieldMax.isVisible()) {
      this.JTextFieldMax.postActionEvent();
      while (this.JTextFieldMax.getActionListeners().length > 0) {
        this.JTextFieldMax.removeActionListener(this.JTextFieldMax.getActionListeners()[0]);
      }
      this.JTextFieldMax.setVisible(false);
      this.Ctrl.requestFocus();
    }

    if (this.Ctrl.isPrecise() && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.getClickCount() > 1 && this.TheLeft_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {
      this.openPrecisionPopup(this.LEFT_POLYGON, new Point(MouseEvent_Arg.getX(), MouseEvent_Arg.getY()));
      return;
    } else if (this.Ctrl.isPrecise() && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.getClickCount() > 1 && this.TheRight_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {
      this.openPrecisionPopup(this.RIGHT_POLYGON, new Point(
              MouseEvent_Arg.getX(), MouseEvent_Arg.getY()));
      return;
    } // Clic the first number to drag it and change the SegmentCount
    else if (this.RectFirstLabel != null && MouseEvent_Arg.getClickCount() == 1 && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && this.RectFirstLabel.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && (MouseEvent_Arg.isShiftDown() || this.Ctrl.isIntSegments())) {
      this.Dragging = BiSliderPresentation.SEGMENT_SIZE_INT;

    } else if (this.RectFirstLabel != null && MouseEvent_Arg.getClickCount() == 1 && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && this.RectFirstLabel.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && !MouseEvent_Arg.isShiftDown()) {
      this.Dragging = BiSliderPresentation.SEGMENT_SIZE;
    } else if (this.TheLeft_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && !MouseEvent_Arg.isAltDown() && (MouseEvent_Arg.isShiftDown() || Ctrl.isAlignOnSegments() )) {

      this.Dragging = BiSliderPresentation.SHIFT_LEFT_RULER;

      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } else if (this.TheLeft_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && !MouseEvent_Arg.isShiftDown() && !MouseEvent_Arg.isAltDown() && (MouseEvent_Arg.isControlDown() || this.Ctrl.isIntValues())) {

      this.Dragging = BiSliderPresentation.CTRL_LEFT_RULER;

      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    }// The user want to modify the minimum by draging the left triangle
    else if (this.TheLeft_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && !MouseEvent_Arg.isAltDown() && !MouseEvent_Arg.isShiftDown()) {

      this.Dragging = BiSliderPresentation.LEFT_RULER;

      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } // the user wants to modify the maximum by draging the left triangle
    else if (this.TheRight_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && !MouseEvent_Arg.isShiftDown() && !MouseEvent_Arg.isAltDown() && (MouseEvent_Arg.isControlDown() || this.Ctrl.isIntValues())) {
      this.Dragging = BiSliderPresentation.CTRL_RIGHT_RULER;
      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    }// the user wants to modify the maximum by draging the left triangle
    else if (this.TheRight_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && !MouseEvent_Arg.isAltDown() && (MouseEvent_Arg.isShiftDown() || Ctrl.isAlignOnSegments() )) {
      this.Dragging = BiSliderPresentation.SHIFT_RIGHT_RULER;

      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } // the user want to modify the maximum by draging the left triangle
    else if (this.TheRight_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && !MouseEvent_Arg.isAltDown() && !MouseEvent_Arg.isShiftDown()) {
      this.Dragging = BiSliderPresentation.RIGHT_RULER;

      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } // The user may want to drag the segment (but stay aligned on int)
    else if (MouseEvent_Arg.getX() > this.LeftValue && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.getX() < this.RightValue && MouseEvent_Arg.getY() > this.marginRulerTop && MouseEvent_Arg.getY() < this.marginRulerTop + this.RulerHeight && !MouseEvent_Arg.isAltDown() && !MouseEvent_Arg.isShiftDown() && (MouseEvent_Arg.isControlDown() || this.Ctrl.isIntValues())) {
      this.Dragging = BiSliderPresentation.CTRL_SEGMENT;

      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    }// The user may want to drag the segment (but stay aligned on graduations)
    else if (MouseEvent_Arg.getX() > this.LeftValue && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.getX() < this.RightValue && MouseEvent_Arg.getY() > this.marginRulerTop && MouseEvent_Arg.getY() < this.marginRulerTop + this.RulerHeight && !MouseEvent_Arg.isAltDown() && (MouseEvent_Arg.isShiftDown() || Ctrl.isAlignOnSegments() )) {
      this.Dragging = BiSliderPresentation.SHIFT_SEGMENT;

      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } // The user may want to drag the segment.
    else if (MouseEvent_Arg.getX() > this.LeftValue && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.getX() < this.RightValue && MouseEvent_Arg.getY() > this.marginRulerTop && MouseEvent_Arg.getY() < this.marginRulerTop + this.RulerHeight && !MouseEvent_Arg.isAltDown() && !MouseEvent_Arg.isShiftDown()) {
      this.Dragging = BiSliderPresentation.SEGMENT;

      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } // The user may want to drag the segment.
    else if (this.TheRight_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.isAltDown()) {
      this.Dragging = BiSliderPresentation.ALT_RIGHT_RULER;

      this.Center = (this.Ctrl.getMinimumColoredValue() + this.Ctrl.getMaximumColoredValue()) / 2;
      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } // The user may want to drag the segment.
    else if (this.TheLeft_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY()) && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.isAltDown()) {
      this.Dragging = BiSliderPresentation.ALT_LEFT_RULER;

      this.Center = (this.Ctrl.getMinimumColoredValue() + this.Ctrl.getMaximumColoredValue()) / 2;
      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } // Double click in a new segment with SHIFT pressed for
    // concatenation of the segment
    else if (MouseEvent_Arg.isShiftDown() && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.getClickCount() > 1 && MouseEvent_Arg.getX() > BiSliderPresentation.MARGIN_RULER_LEFT && MouseEvent_Arg.getX() < BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth && MouseEvent_Arg.getY() > this.marginRulerTop && MouseEvent_Arg.getY() < this.marginRulerTop + this.RulerHeight) {

      int d1 = MouseEvent_Arg.getX() - BiSliderPresentation.MARGIN_RULER_LEFT;
      int GraduationCount = (int) Math.floor(d1 / this.GraduationWidth);

      double Min = this.Ctrl.getMinimumValue() + (float) ((GraduationCount * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);
      double Max = this.Ctrl.getMinimumValue() + (float) (((GraduationCount + 1) * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);

      // because the last segment is maybe smaller than a regular one
      if (Max > this.Ctrl.getMaximumValue()) {
        Max = this.Ctrl.getMaximumValue();
      }

      
      if (Min < this.Ctrl.getMinimumColoredValue()) {
        Ctrl.prepareJump();
        this.Ctrl.setColoredValues(Min, this.Ctrl.getMaximumColoredValue());
        Ctrl.notifyJump();
      } else if (Max > this.Ctrl.getMaximumColoredValue()) {
        Ctrl.prepareJump();
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(), Max);
        Ctrl.notifyJump();
      }

      // Like that the user doesn't have to release the mouse and reclick to drag the
      // segmentS
      this.Dragging = BiSliderPresentation.SHIFT_SEGMENT;
      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();

      this.JComponent1.repaint();
    } // Double click in a new segment
    else if (MouseEvent_Arg.getClickCount() > 1 && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.getX() > BiSliderPresentation.MARGIN_RULER_LEFT && MouseEvent_Arg.getX() < BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth && MouseEvent_Arg.getY() > this.marginRulerTop && MouseEvent_Arg.getY() < this.marginRulerTop + this.RulerHeight) {

      int d1 = MouseEvent_Arg.getX() - BiSliderPresentation.MARGIN_RULER_LEFT;
      int GraduationCount = (int) Math.floor(d1 / this.GraduationWidth);

      double Min = this.Ctrl.getMinimumValue() + (float) ((GraduationCount * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);
      double Max = this.Ctrl.getMinimumValue() + (float) (((GraduationCount + 1) * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);

      // because the last segment is maybe smaller than a regular one
      if (Max > this.Ctrl.getMaximumValue()) {
        Max = this.Ctrl.getMaximumValue();
      }

      Ctrl.prepareJump();
      this.Ctrl.setColoredValues(Min, Max);
      Ctrl.notifyJump();

      // Like that the user doesn't have to release the mouse to drag the segment
      if (!this.Ctrl.isIntValues())
        this.Dragging = BiSliderPresentation.SEGMENT;
      else
        this.Dragging = BiSliderPresentation.CTRL_SEGMENT;
      this.DeplBef = MouseEvent_Arg.getX() - this.LeftValue;
      this.DeplAft = this.RightValue - MouseEvent_Arg.getX();
      this.JComponent1.repaint();
    
    //grab a new segment 
    } else if (MouseEvent_Arg.getClickCount() == 1 && MouseEvent_Arg.getButton() == MouseEvent.BUTTON1 && MouseEvent_Arg.getX() > BiSliderPresentation.MARGIN_RULER_LEFT && MouseEvent_Arg.getX() < BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth && MouseEvent_Arg.getY() > this.marginRulerTop && MouseEvent_Arg.getY() < this.marginRulerTop + this.RulerHeight) {
      int d1 = MouseEvent_Arg.getX() - BiSliderPresentation.MARGIN_RULER_LEFT;
      int GraduationCount = (int) Math.floor(d1 / this.GraduationWidth);

      double Min = this.Ctrl.getMinimumValue() + (float) ((GraduationCount * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);
      double Max = this.Ctrl.getMinimumValue() + (float) (((GraduationCount + 1) * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);

      Ctrl.notifyGrabSegment(Min, Max, d1, GraduationCount);
      this.Dragging = BiSliderPresentation.DRAG_SEGMENT;
    } else {
      return;
    }
    MouseEvent_Arg.consume();
  } // mousePressed()

  /**
   * Method called by the awt-swing mechanism when the user drag his mouse
   * over this component
   * 
   * @param MouseEvent_Arg
   *            the mouse event generatedby the awt-swing mechanism
   */
  public void mouseDragged(MouseEvent MouseEvent_Arg) {

    // System.out.println("last="+LastFiveEvents.elementAt(LastFiveEvents.size()-1));
    // System.out.println("new="+MouseEvent_Arg.getPoint());
    if ((this.LastFiveEvents.elementAt(0)).x != MouseEvent_Arg.getX() || (this.LastFiveEvents.elementAt(0)).y != MouseEvent_Arg.getY()) {
      this.LastFiveEvents.add(0, (Point) MouseEvent_Arg.getPoint().clone());
    }
    if (this.LastFiveEvents.size() > 5) {
      this.LastFiveEvents.removeElementAt(5);
    }

    // In case the event is outside the bislider area
    if (MouseEvent_Arg.getX() > this.JComponent1.getSize().width) {
      MouseEvent_Arg.translatePoint(this.JComponent1.getSize().width - MouseEvent_Arg.getX(), 0);
    }
    if (MouseEvent_Arg.getX() < 0) {
      MouseEvent_Arg.translatePoint(MouseEvent_Arg.getX(), 0);
    }
    if (MouseEvent_Arg.getY() > this.JComponent1.getSize().height) {
      MouseEvent_Arg.translatePoint(0, this.JComponent1.getSize().height - MouseEvent_Arg.getY());
    }
    if (MouseEvent_Arg.getY() < 0) {
      MouseEvent_Arg.translatePoint(0, MouseEvent_Arg.getY());
    }

    // if (MouseX != MouseEvent_Arg.getX())
    // System.err.println("we changed mouseX from "+MouseX+" to
    // "+MouseEvent_Arg.getX());
    int MouseX = MouseEvent_Arg.getX();

    // change the dragging mode during the drag !
    if (this.Dragging == BiSliderPresentation.LEFT_RULER && MouseEvent_Arg.isAltDown()) {
      this.Dragging = BiSliderPresentation.ALT_LEFT_RULER;
      this.Center = (this.Ctrl.getMinimumColoredValue() + this.Ctrl.getMaximumColoredValue()) / 2;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.RIGHT_RULER && MouseEvent_Arg.isAltDown()) {
      this.Dragging = BiSliderPresentation.ALT_RIGHT_RULER;
      this.Center = (this.Ctrl.getMinimumColoredValue() + this.Ctrl.getMaximumColoredValue()) / 2;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.LEFT_RULER && MouseEvent_Arg.isShiftDown()) {
      this.Dragging = BiSliderPresentation.SHIFT_LEFT_RULER;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.RIGHT_RULER && MouseEvent_Arg.isShiftDown()) {
      this.Dragging = BiSliderPresentation.SHIFT_RIGHT_RULER;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.RIGHT_RULER && MouseEvent_Arg.isControlDown()) {
      this.Dragging = BiSliderPresentation.CTRL_RIGHT_RULER;
    }    
    
    // idem
    if (this.Dragging == BiSliderPresentation.SHIFT_SEGMENT && !MouseEvent_Arg.isShiftDown() && !this.Ctrl.isIntValues()) {
      this.Dragging = BiSliderPresentation.SEGMENT;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.SEGMENT && MouseEvent_Arg.isShiftDown()) {
      this.Dragging = BiSliderPresentation.SHIFT_SEGMENT;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.SEGMENT_SIZE && MouseEvent_Arg.isShiftDown()) {
      this.Dragging = BiSliderPresentation.SEGMENT_SIZE_INT;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.SEGMENT_SIZE_INT && !MouseEvent_Arg.isShiftDown() && !this.Ctrl.isIntSegments()) {
      this.Dragging = BiSliderPresentation.SEGMENT_SIZE;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.ALT_LEFT_RULER && !MouseEvent_Arg.isAltDown()) {
      this.Dragging = BiSliderPresentation.LEFT_RULER;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.ALT_RIGHT_RULER && !MouseEvent_Arg.isAltDown()) {
      this.Dragging = BiSliderPresentation.RIGHT_RULER;
    }

    // idem
    if (this.Dragging == BiSliderPresentation.SHIFT_LEFT_RULER && !MouseEvent_Arg.isShiftDown() && !this.Ctrl.isIntValues()) {
      this.Dragging = BiSliderPresentation.LEFT_RULER;
    }
    
    // idem
    if (this.Dragging == BiSliderPresentation.LEFT_RULER && MouseEvent_Arg.isControlDown()) {
      this.Dragging = BiSliderPresentation.CTRL_LEFT_RULER;
    }     

    // idem
    if (this.Dragging == BiSliderPresentation.SHIFT_RIGHT_RULER && !MouseEvent_Arg.isShiftDown() && !this.Ctrl.isIntValues()) {
      this.Dragging = BiSliderPresentation.RIGHT_RULER;
    }
    
    // change the Segment size !
    if (this.Dragging == BiSliderPresentation.SEGMENT_SIZE) {
      double SegSize = (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) * (MouseX - BiSliderPresentation.MARGIN_RULER_LEFT) / this.RulerWidth;
      SegSize = Math.min(SegSize, this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue());
      if (SegSize > 0) {
        this.Ctrl.setSegmentSize(SegSize);
      }
    } else if (this.Dragging == BiSliderPresentation.SEGMENT_SIZE_INT) {
      double SegSize = Math.round((this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) * (MouseX - BiSliderPresentation.MARGIN_RULER_LEFT) / this.RulerWidth);
      if (SegSize > 0 && SegSize < (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) {
        this.Ctrl.setSegmentSize(SegSize);
      }
    } // drag the minimum value with centering
    else if (this.Dragging == BiSliderPresentation.ALT_LEFT_RULER) {
      double NewValue = this.Ctrl.getMinimumValue() + ((MouseX - this.DeplBef - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;

      if (NewValue > this.Center) {
        NewValue = this.Center;
      }

      if (NewValue < this.Ctrl.getMinimumValue()) {
        NewValue = this.Ctrl.getMinimumValue();
      }

      if (this.Center * 2 - NewValue > this.Ctrl.getMaximumValue()) {
        NewValue = 2 * this.Center - this.Ctrl.getMaximumValue();
      }

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(NewValue, 2 * this.Center - NewValue);

      this.JComponent1.repaint();
    } // drag the maximum value with centering
    else if (this.Dragging == BiSliderPresentation.ALT_RIGHT_RULER) {
      double NewValue = this.Ctrl.getMinimumValue() + ((MouseX + this.DeplAft - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;

      if (NewValue < this.Center) {
        NewValue = this.Center;
      }

      if (NewValue > this.Ctrl.getMaximumValue()) {
        NewValue = this.Ctrl.getMaximumValue();
      }

      if (this.Center * 2 - NewValue < this.Ctrl.getMinimumValue()) {
        NewValue = 2 * this.Center - this.Ctrl.getMinimumValue();
      }

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(this.Center * 2 - NewValue, NewValue);
      this.JComponent1.repaint();
    } // Drag the left triangle
    else if (this.Dragging == BiSliderPresentation.LEFT_RULER) {
      if (this.Ctrl.isPrecise() && this.isLastFiveEventsUpDown()) {
        this.openPrecisionPopup(this.LEFT_POLYGON, new Point(
                MouseEvent_Arg.getX(), MouseEvent_Arg.getY()));
        return;
      }

      if (MouseX - this.DeplBef > this.RightValue) {
        this.Ctrl.setColoredValues(this.Ctrl.getMaximumColoredValue(),
                this.Ctrl.getMaximumColoredValue());
        this.JComponent1.repaint();
        return;
      } else if (MouseX - this.DeplBef < BiSliderPresentation.MARGIN_RULER_LEFT) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue(),
                this.Ctrl.getMaximumColoredValue());
        this.JComponent1.repaint();
        return;
      }

      // change the value and call repaint to display the modifications
      double NewMinValue = this.Ctrl.getMinimumValue() + ((MouseX - this.DeplBef - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;
      NewMinValue = Math.min(NewMinValue, this.Ctrl.getMaximumColoredValue());
      this.Ctrl.setColoredValues(NewMinValue, this.Ctrl.getMaximumColoredValue());

      this.JComponent1.repaint();
    // JComponent1.paintImmediately(0,0, JComponent1.getWidth(),
    // JComponent1.getHeight());
    } // Drag the left triangle aligned on graduation
    else if (this.Dragging == BiSliderPresentation.SHIFT_LEFT_RULER) {
      double Size = (float) (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());

      if (MouseX - this.DeplBef > this.RightValue) {
        this.Ctrl.setColoredValues(this.Ctrl.getMaximumColoredValue(),
                this.Ctrl.getMaximumColoredValue());
        this.JComponent1.repaint();
        return;
      } else if (MouseX - this.DeplBef < BiSliderPresentation.MARGIN_RULER_LEFT) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue(),
                this.Ctrl.getMaximumColoredValue());
        this.JComponent1.repaint();
        return;
      }

      double DMin = ((MouseX - this.DeplBef - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;
      int NewMin = (int) Math.round(DMin / this.Ctrl.getSegmentSize());
      while (this.Ctrl.getMinimumValue() + (this.Ctrl.getSegmentSize() * NewMin) < this.Ctrl.getMinimumValue()) {
        NewMin++;
      }

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue() + (this.Ctrl.getSegmentSize() * NewMin), this.Ctrl.getMaximumColoredValue());

      this.JComponent1.repaint();
    // JComponent1.paintImmediately(0,0, JComponent1.getWidth(),
    // JComponent1.getHeight());
    } // Drag the left triangle aligned on int
    else if (this.Dragging == BiSliderPresentation.CTRL_LEFT_RULER) {
      double Size = (float) (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());

      if (MouseX - this.DeplBef > this.RightValue) {
        this.Ctrl.setColoredValues(this.Ctrl.getMaximumColoredValue(),
                this.Ctrl.getMaximumColoredValue());
        this.JComponent1.repaint();
        return;
      } else if (MouseX - this.DeplBef < BiSliderPresentation.MARGIN_RULER_LEFT) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue(),
                this.Ctrl.getMaximumColoredValue());
        this.JComponent1.repaint();
        return;
      }

      double DMin = ((MouseX - this.DeplBef - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;
      int NewMin = (int) Math.round(DMin);
      /*while (this.Ctrl.getMinimumValue() + (this.Ctrl.getSegmentSize() * NewMin) < this.Ctrl.getMinimumValue()) {
        NewMin++;
      }*/

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue() + ( NewMin), this.Ctrl.getMaximumColoredValue());

      this.JComponent1.repaint();
    // JComponent1.paintImmediately(0,0, JComponent1.getWidth(),
    // JComponent1.getHeight());
    }// Drag the right triangle
    else if (this.Dragging == BiSliderPresentation.RIGHT_RULER) {
      if (this.Ctrl.isPrecise() && this.isLastFiveEventsUpDown()) {
        this.openPrecisionPopup(this.RIGHT_POLYGON, new Point(
                MouseEvent_Arg.getX(), MouseEvent_Arg.getY()));
        return;
      }

      if (MouseX + this.DeplAft < this.LeftValue) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
                this.Ctrl.getMinimumColoredValue());
        this.JComponent1.repaint();
        return;
      } else if (MouseX + this.DeplAft > BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
                this.Ctrl.getMaximumValue());
        this.JComponent1.repaint();
        return;
      }

      // change the value and call repaint to display the modifications
      double NewMaxValue = this.Ctrl.getMinimumValue() + ((MouseX + this.DeplAft - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;
      NewMaxValue = Math.max(NewMaxValue, this.Ctrl.getMinimumColoredValue());
      this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
              NewMaxValue);

      this.JComponent1.repaint();
    } // Drag the right triangle aligned on int
    else if (this.Dragging == BiSliderPresentation.CTRL_RIGHT_RULER) {
      double Size = (float) (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());

      if (MouseX + this.DeplAft < this.LeftValue) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
                this.Ctrl.getMaximumValue());
        this.JComponent1.repaint();
        return;
      } else if (MouseX + this.DeplAft > BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
                this.Ctrl.getMaximumValue());
        this.JComponent1.repaint();
        return;
      }

      double DMax = ((MouseX + this.DeplAft + BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;
      int NewMax = (int) Math.round(DMax );
      /*while (this.Ctrl.getMinimumValue() + (this.Ctrl.getSegmentSize() * NewMax) > this.Ctrl.getMaximumValue()) {
        NewMax--;
      }*/

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
              this.Ctrl.getMinimumValue() + ( NewMax));

      this.JComponent1.repaint();
    }// Drag the right triangle aligned on graduation
    else if (this.Dragging == BiSliderPresentation.SHIFT_RIGHT_RULER) {
      double Size = (float) (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());

      if (MouseX + this.DeplAft < this.LeftValue) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
                this.Ctrl.getMaximumValue());
        this.JComponent1.repaint();
        return;
      } else if (MouseX + this.DeplAft > BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
                this.Ctrl.getMaximumValue());
        this.JComponent1.repaint();
        return;
      }

      double DMax = ((MouseX + this.DeplAft + BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;
      int NewMax = (int) Math.round(DMax / this.Ctrl.getSegmentSize());
      while (this.Ctrl.getMinimumValue() + (this.Ctrl.getSegmentSize() * NewMax) > this.Ctrl.getMaximumValue()) {
        NewMax--;
      }

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
              this.Ctrl.getMinimumValue() + (this.Ctrl.getSegmentSize() * NewMax));

      this.JComponent1.repaint();
    } // Drag and drop the segment but align the minimum with a n int
    else if (this.Dragging == BiSliderPresentation.CTRL_SEGMENT) {
      double Size = (float) (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());

      // if the drag goes too left we must stop moving the segment
      if (MouseX - this.DeplBef <= BiSliderPresentation.MARGIN_RULER_LEFT) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue(),
                this.Ctrl.getMinimumValue() + Size);
        this.JComponent1.repaint();
        return;
      } // same at right
      else if (MouseX + this.DeplAft >= BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
        this.Ctrl.setColoredValues(this.Ctrl.getMaximumValue() - Size,
                this.Ctrl.getMaximumValue());
        this.JComponent1.repaint();
        return;
      }

      double DMin = ((MouseX - this.DeplBef - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;
      int NewMin = (int) Math.round(DMin );
      double MagnetEffect = (NewMin) - DMin;

      if (this.Ctrl.getMinimumValue() + DMin + MagnetEffect + Size > this.Ctrl.getMaximumValue()) {
        DMin = this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue() - MagnetEffect - Size;
      }

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue() + DMin + MagnetEffect, this.Ctrl.getMinimumValue() + DMin + MagnetEffect + Size);

      // JComponent1.paintImmediately(0,0, JComponent1.getWidth(),
      // JComponent1.getHeight());
      this.JComponent1.repaint();
    }// Drag and drop the segment but align the minimum with a graduation
    else if (this.Dragging == BiSliderPresentation.SHIFT_SEGMENT) {
      double Size = (float) (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());

      // if the drag goes too left we must stop moving the segment
      if (MouseX - this.DeplBef <= BiSliderPresentation.MARGIN_RULER_LEFT) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue(),
                this.Ctrl.getMinimumValue() + Size);
        this.JComponent1.repaint();
        return;
      } // same at right
      else if (MouseX + this.DeplAft >= BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
        this.Ctrl.setColoredValues(this.Ctrl.getMaximumValue() - Size,
                this.Ctrl.getMaximumValue());
        this.JComponent1.repaint();
        return;
      }

      double DMin = ((MouseX - this.DeplBef - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;
      int NewMin = (int) Math.round(DMin / this.Ctrl.getSegmentSize());
      double MagnetEffect = (this.Ctrl.getSegmentSize() * NewMin) - DMin;

      if (this.Ctrl.getMinimumValue() + DMin + MagnetEffect + Size > this.Ctrl.getMaximumValue()) {
        DMin = this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue() - MagnetEffect - Size;
      }

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue() + DMin + MagnetEffect, this.Ctrl.getMinimumValue() + DMin + MagnetEffect + Size);

      // JComponent1.paintImmediately(0,0, JComponent1.getWidth(),
      // JComponent1.getHeight());
      this.JComponent1.repaint();
    } // Drag and drop the segment
    else if (this.Dragging == BiSliderPresentation.SEGMENT) {
      double Size = (float) (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());

      // if the drag goes too left we must stop moving the segment
      if (MouseX - this.DeplBef <= BiSliderPresentation.MARGIN_RULER_LEFT) {
        this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue(),
                this.Ctrl.getMinimumValue() + Size);
        this.JComponent1.repaint();
        return;
      } // same at right
      else if (MouseX + this.DeplAft >= BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
        this.Ctrl.setColoredValues(this.Ctrl.getMaximumValue() - Size,
                this.Ctrl.getMaximumValue());
        this.JComponent1.repaint();
        return;
      }

      double DMin = ((MouseX - this.DeplBef - BiSliderPresentation.MARGIN_RULER_LEFT) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue())) / this.RulerWidth;

      // change the value and call repaint to display the modifications
      this.Ctrl.setColoredValues(this.Ctrl.getMinimumValue() + DMin,
              this.Ctrl.getMinimumValue() + DMin + Size);

      // JComponent1.paintImmediately(0,0, JComponent1.getWidth(),
      // JComponent1.getHeight());
      this.JComponent1.repaint();
    } else if (this.Dragging == BiSliderPresentation.DRAG_SEGMENT){
      int d1 = MouseEvent_Arg.getX() - BiSliderPresentation.MARGIN_RULER_LEFT;
      int GraduationCount = (int) Math.floor(d1 / this.GraduationWidth);

      double Min = this.Ctrl.getMinimumValue() + (float) ((GraduationCount * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);
      double Max = this.Ctrl.getMinimumValue() + (float) (((GraduationCount + 1) * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);

      Ctrl.notifyDragSegment(Min, Max, d1, GraduationCount);

    } else {
      return;
    }

    MouseEvent_Arg.consume();
  } // mouseDragged()

  /**
   * Method called by the awt-swing mechanism when the user release the mouse
   * button over this component
   * 
   * @param MouseEvent_Arg
   *            the mouse event generatedby the awt-swing mechanism
   */
  public void mouseReleased(MouseEvent MouseEvent_Arg) {
    if (this.Dragging == BiSliderPresentation.DRAG_SEGMENT){
      int d1 = MouseEvent_Arg.getX() - BiSliderPresentation.MARGIN_RULER_LEFT;
      int GraduationCount = (int) Math.floor(d1 / this.GraduationWidth);

      double Min = this.Ctrl.getMinimumValue() + (float) ((GraduationCount * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);
      double Max = this.Ctrl.getMinimumValue() + (float) (((GraduationCount + 1) * this.GraduationWidth) * (this.Ctrl.getMaximumValue() - this.Ctrl.getMinimumValue()) / this.RulerWidth);

      Ctrl.notifyReleaseSegment(Min, Max, d1, GraduationCount);
    }

    if (this.Dragging != BiSliderPresentation.PRECISE_LEFT_RULER && this.Dragging != BiSliderPresentation.PRECISE_RIGHT_RULER) {
      this.Dragging = BiSliderPresentation.NONE;
    }

    this.JComponent1.repaint();

    this.Ctrl.setColoredValues(this.Ctrl.getMinimumColoredValue(),
            this.Ctrl.getMaximumColoredValue());


  } // mouseReleased()

  /**
   * Method called by the awt-swing mechanism when the user move his mouse
   * over this component
   * 
   * @param MouseEvent_Arg
   *            the mouse event generatedby the awt-swing mechanism
   */
  public void mouseMoved(MouseEvent MouseEvent_Arg) {
    int OldMouseUnder = this.MouseUnder;
    Rectangle2D OldRectangleSegment = this.RectangleSegment;

    // for mouseOver between the triangles
    Rectangle Rect1 = this.TheRight_Polygon.getBounds();
    Rectangle Rect2 = this.TheLeft_Polygon.getBounds();
    Rectangle2D Rectangle2D1 = Rect1.createUnion(Rect2);

    // for segment mouse over
    int d1 = MouseEvent_Arg.getX() - BiSliderPresentation.MARGIN_RULER_LEFT;
    int GraduationCount = (int) Math.floor(d1 / this.GraduationWidth);

    int LeftSegment = BiSliderPresentation.MARGIN_RULER_LEFT + (int) ((GraduationCount * this.GraduationWidth));
    int RightSegment = BiSliderPresentation.MARGIN_RULER_LEFT + (int) (((GraduationCount + 1) * this.GraduationWidth));
    if (RightSegment > BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
      RightSegment = BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth;
    }

    /*
     * double ValuesWidth = Ctrl.getMaximumValue() - Ctrl.getMinimumValue();
     * int d1 = MouseEvent_Arg.getX() - MARGIN_RULER_LEFT; double d2 =
     * ((double)d1 * (Ctrl.getMaximumValue() - Ctrl.getMinimumValue())) /
     * RulerWidth; int d4 = ((int)(Ctrl.getMaximumValue() -
     * Ctrl.getMinimumValue()) / Ctrl.getSegmentCount()); d1 = (int)(d2); d1 =
     * (int)Ctrl.getMinimumValue() + (d1 - (d1 % d4)); int LeftSegment =
     * (int)(MARGIN_RULER_LEFT + (RulerWidth * (d1 -
     * Ctrl.getMinimumValue())) / ValuesWidth); int RightSegment =
     * (int)(MARGIN_RULER_LEFT + (RulerWidth * (d1 + d4 -
     * Ctrl.getMinimumValue())) / ValuesWidth);
     */
    if (MouseEvent_Arg.getX() >= BiSliderPresentation.MARGIN_RULER_LEFT && MouseEvent_Arg.getX() <= BiSliderPresentation.MARGIN_RULER_LEFT + this.RulerWidth) {
      this.RectangleSegment = new Rectangle(LeftSegment + 1,
              this.marginRulerTop + 1, RightSegment - LeftSegment - 2,
              this.RulerHeight - 2);
    } else {
      this.RectangleSegment = null;
    }

    if (MouseEvent_Arg.isShiftDown() && this.RectangleSegment != null) {
      this.RectangleSegment = Rectangle2D1.createUnion(this.RectangleSegment);
    }

    if (this.TheRight_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {
      this.MouseUnder = this.RIGHT_POLYGON;
    } else if (this.TheLeft_Polygon.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {
      this.MouseUnder = this.LEFT_POLYGON;
    } else if (Rectangle2D1.contains(MouseEvent_Arg.getX(), MouseEvent_Arg.getY())) {
      this.MouseUnder = this.SELECTION;
    } else if (this.RectFirstLabel != null && this.RectFirstLabel.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {
      this.MouseUnder = this.FIRST_LABEL;
    } else if (this.RectLastLabel != null && this.RectLastLabel.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {
      this.MouseUnder = this.LAST_LABEL;
    } else if (this.RectangleSegment != null && this.RectangleSegment.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {
      this.MouseUnder = BiSliderPresentation.SEGMENT;
    } else {
      this.MouseUnder = this.NOTHING;
    }

    if (this.MouseUnder != OldMouseUnder) {
      this.JComponent1.repaint();
    }

    if (OldRectangleSegment != null && this.MouseUnder == BiSliderPresentation.SEGMENT && !OldRectangleSegment.equals(this.RectangleSegment)) {
      this.JComponent1.repaint();
    }
  } // mouseMoved()


  /**
   * wheel actions handler
   */
  public void mouseWheelMoved(MouseWheelEvent event) {
    double min = this.Ctrl.getMinimumColoredValue() - event.getWheelRotation() * this.Ctrl.getSegmentSize();
    double max = this.Ctrl.getMaximumColoredValue() - event.getWheelRotation() * this.Ctrl.getSegmentSize();
    if (event.getModifiers()!=0){
      min = this.Ctrl.getMinimumColoredValue() + event.getWheelRotation() * this.Ctrl.getSegmentSize();
      max = this.Ctrl.getMaximumColoredValue() + event.getWheelRotation() * this.Ctrl.getSegmentSize();
    }

    if (min < this.Ctrl.getMinimumValue()) {
      min = this.Ctrl.getMinimumValue();
      max = min + (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());
    }
    if (max > this.Ctrl.getMaximumValue()) {
      max = this.Ctrl.getMaximumValue();
      min = max - (this.Ctrl.getMaximumColoredValue() - this.Ctrl.getMinimumColoredValue());
    }
    this.Ctrl.setColoredValues(min, max);
  }


  /**
   * analyse the five last mouse position and tell if it's really following a
   * vertical direction
   * 
   * @return The lastFiveEventsUpDown value
   */
  private boolean isLastFiveEventsUpDown() {
    if (this.LastFiveEvents.size() > 4) {
      Point PointM0 = this.LastFiveEvents.elementAt(0);
      Point PointM1 = this.LastFiveEvents.elementAt(1);
      Point PointM2 = this.LastFiveEvents.elementAt(2);
      Point PointM3 = this.LastFiveEvents.elementAt(3);
      Point PointM4 = this.LastFiveEvents.elementAt(4);
      int D1x = PointM0.x - PointM1.x;
      int D1y = PointM0.y - PointM1.y;
      int D2x = PointM0.x - PointM2.x;
      int D2y = PointM0.y - PointM2.y;
      int D3x = PointM0.x - PointM3.x;
      int D3y = PointM0.y - PointM3.y;
      int D4x = PointM0.x - PointM4.x;
      int D4y = PointM0.y - PointM4.y;

      return ((Math.abs(D1x) < 2 && Math.abs(D1y) > 5) || (Math.abs(D2x) < 2 && Math.abs(D2y) > 5) || (Math.abs(D3x) < 2 && Math.abs(D3y) > 5) || (Math.abs(D4x) < 2 && Math.abs(D4y) > 5));
    } else {
      return false;
    }
  } // isLastFiveEventsUpDown()


  /**
   * open a JSlider in popup to precise the value
   * 
   * @param Thumb_Arg
   *            Description of the Parameter
   * @param Point_Arg
   *            Description of the Parameter
   */
  private void openPrecisionPopup(int Thumb_Arg, Point Point_Arg) {

    this.JSlider1.setValue(0);
    this.JLabel1.setBackground(this.Ctrl.getBackground());
    this.JLabel1.setForeground(this.Ctrl.getForeground());
    this.JSlider1.setBackground(this.Ctrl.getSliderBackground());
    Hashtable<Integer, JLabel> Hashtable1 = new Hashtable<Integer, JLabel>();
    DecimalFormat DecimalFormater = this.Ctrl.getDecimalFormater();

    PopupFactory PopupFactory1 = PopupFactory.getSharedInstance();
    Point Point1 = (Point) Point_Arg.clone();
    SwingUtilities.convertPointToScreen(Point1, this.Ctrl);
    if (Thumb_Arg == this.RIGHT_POLYGON) {
      this.JLabel1.setText(this.Ctrl.getDecimalFormater().format(
              this.Ctrl.getMaximumColoredValue()));
    } else if (Thumb_Arg == this.LEFT_POLYGON) {
      this.JLabel1.setText(this.Ctrl.getDecimalFormater().format(
              this.Ctrl.getMinimumColoredValue()));
    }

    Rectangle limits = Ctrl.getGraphicsConfiguration().getBounds();

    //Dimension ScreenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
    int CenteredY = Point1.y - this.JSlider1.getPreferredSize().height / 2;

    if (CenteredY + this.JPanel1.getPreferredSize().height > limits.y+limits.height)
      CenteredY = limits.y+limits.height - this.JPanel1.getPreferredSize().height;
    if (Point1.x - 8 + this.JPanel1.getPreferredSize().width > limits.x+limits.width)
      Point1.x = limits.x+limits.width - this.JPanel1.getPreferredSize().width+8;

    if (CenteredY < 0) {
      CenteredY = 0;
    }

    final Popup popup1 = PopupFactory1.getPopup(Ctrl, JPanel1, Point1.x - 8, CenteredY);

    double MiddleVal = this.Ctrl.getMinimumColoredValue();
    if (Thumb_Arg == this.RIGHT_POLYGON) {
      MiddleVal = this.Ctrl.getMaximumColoredValue();
    }

    double Amplitude = this.Ctrl.getSegmentSize() * Ctrl.getPrecisionAmplitudeRatio();
    if (Thumb_Arg == this.RIGHT_POLYGON && MiddleVal - Amplitude < this.Ctrl.getMinimumColoredValue()) {
      MiddleVal = this.Ctrl.getMinimumColoredValue() + Amplitude;
    } else if (Thumb_Arg == this.LEFT_POLYGON && MiddleVal + Amplitude > this.Ctrl.getMaximumColoredValue()) {
      MiddleVal = this.Ctrl.getMaximumColoredValue() - Amplitude;
    } else if (MiddleVal - Amplitude < this.Ctrl.getMinimumValue()) {
      MiddleVal = this.Ctrl.getMinimumValue() + Amplitude;
    } else if (MiddleVal + Amplitude > this.Ctrl.getMaximumValue()) {
      MiddleVal = this.Ctrl.getMaximumValue() - Amplitude;
    }

    for (int i = -100; i <= 100; i += 20) {
      final double Val = MiddleVal + (i) / 100d * Amplitude;
      final int li = i+100;

      Hashtable1.put(new Integer(i), new JLabel(DecimalFormater.format(Val)){
        @Override
        public void paint(Graphics g){
          if (Ctrl.UnitPainterSupport1.getPainterListenerNumber()>0){
            //g.setColor(Color.yellow);
            //g.fillRect(0, 0, 20, 20);
            //System.out.println("paint label "+Val+" "+li+" "+getBounds());
            Ctrl.UnitPainterSupport1.firePaintTick(popup1, (Graphics2D)g, Val, li, Ctrl.getForeground(), getBounds());
          } else
            super.paint(g);
        }
      });
    }

    this.JSlider1.setLabelTable(Hashtable1);
    this.JSlider1.validate();
   
    popup1.show();

    this.JSlider1.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseReleased(MouseEvent MouseEvent_Arg) {
        popup1.hide();
        BiSliderPresentation.this.JSlider1.removeMouseListener(this);
        double MiddleVal = BiSliderPresentation.this.PreciseOpenedValue;

        double Amplitude = BiSliderPresentation.this.Ctrl.getSegmentSize()* Ctrl.getPrecisionAmplitudeRatio();
        if (BiSliderPresentation.this.Dragging == BiSliderPresentation.PRECISE_RIGHT_RULER && MiddleVal - Amplitude < BiSliderPresentation.this.Ctrl.getMinimumColoredValue()) {
          MiddleVal = BiSliderPresentation.this.Ctrl.getMinimumColoredValue() + Amplitude;
        } else if (BiSliderPresentation.this.Dragging == BiSliderPresentation.PRECISE_LEFT_RULER && MiddleVal + Amplitude > BiSliderPresentation.this.Ctrl.getMaximumColoredValue()) {
          MiddleVal = BiSliderPresentation.this.Ctrl.getMaximumColoredValue() - Amplitude;
        } else if (MiddleVal - Amplitude < BiSliderPresentation.this.Ctrl.getMinimumValue()) {
          MiddleVal = BiSliderPresentation.this.Ctrl.getMinimumValue() + Amplitude;
        } else if (MiddleVal + Amplitude > BiSliderPresentation.this.Ctrl.getMaximumValue()) {
          MiddleVal = BiSliderPresentation.this.Ctrl.getMaximumValue() - Amplitude;
        }

        double Val = MiddleVal + (BiSliderPresentation.this.JSlider1.getValue()) / 100d * Amplitude;

        if (BiSliderPresentation.this.Dragging == BiSliderPresentation.PRECISE_RIGHT_RULER) {
          BiSliderPresentation.this.Ctrl.setMaximumColoredValue(Val);
        } else if (BiSliderPresentation.this.Dragging == BiSliderPresentation.PRECISE_LEFT_RULER) {
          BiSliderPresentation.this.Ctrl.setMinimumColoredValue(Val);
        }

        BiSliderPresentation.this.Ctrl.repaint();
      }
    });

    // This is the only trick I found to make the bislider lost the drag
    // focus
    // and jump directly to the JSlider so the user doesn't have to release
    // and re-click !
    try {
      Robot Robot1 = new Robot();
      Robot1.mouseRelease(InputEvent.BUTTON1_MASK);
      Robot1.mouseMove(Point1.x, CenteredY + this.JSlider1.getPreferredSize().height / 2);
      Robot1.mousePress(InputEvent.BUTTON1_MASK);
    } catch (AWTException AWTException_Arg) {
      AWTException_Arg.printStackTrace();
    }

    if (Thumb_Arg == this.RIGHT_POLYGON) {
      this.Dragging = BiSliderPresentation.PRECISE_RIGHT_RULER;
      this.PreciseOpenedValue = this.Ctrl.getMaximumColoredValue();
    } else if (Thumb_Arg == this.LEFT_POLYGON) {
      this.Dragging = BiSliderPresentation.PRECISE_LEFT_RULER;
      this.PreciseOpenedValue = this.Ctrl.getMinimumColoredValue();
    }
  } // openPrecisionPopup()

  /**
   * Method called by the awt-swing mechanism when something/someone resize
   * this component
   * 
   * @param ComponentEvent_Arg
   *            Description of the Parameter
   */
  public void componentResized(ComponentEvent ComponentEvent_Arg) {
    this.RulerHeight = this.Ctrl.getHeight() - this.marginRulerTop - BiSliderPresentation.MARGIN_RULER_BOTTOM;
    this.Ctrl.repaint();
  } // componentResized()

  /**
   * Description of the Method
   * 
   * @param e
   *            Description of the Parameter
   */
  public void componentShown(ComponentEvent e) {
  }

  /**
   * Description of the Method
   * 
   * @param e
   *            Description of the Parameter
   */
  public void componentHidden(ComponentEvent e) {
  }

  /**
   * Description of the Method
   * 
   * @param e
   *            Description of the Parameter
   */
  public void componentMoved(ComponentEvent e) {
  }


  /**
   * change the minimum value of the slider
   * 
   * @param NewValue_Arg
   *            Description of the Parameter
   */
  protected void changeMinValue(String NewValue_Arg) {
    if (!this.Ctrl.getUnit().equals("alpha")) {
      try {
        double NV = Double.parseDouble(NewValue_Arg);

        if (NV <= this.Ctrl.getMaximumValue()-Ctrl.getMinimumColoredSize()) {
          this.Ctrl.setMinimumValue(NV);
        }

        this.JComponent1.repaint();
      } catch (Exception Exception_Arg) {
        Exception_Arg.printStackTrace();
      }
    } else {
      double NV = NewValue_Arg.toUpperCase().charAt(0) - 'A';
      if (NV <= this.Ctrl.getMaximumValue()-Ctrl.getMinimumColoredSize()) {
        this.Ctrl.setMinimumValue(NV);
        this.Ctrl.setSegmentSize(1);
      }
    }
    this.Ctrl.requestFocus();
  } // changeMinValue()


  /**
   * change the maximum value of the slider
   * 
   * @param NewValue_Arg
   *            Description of the Parameter
   */
  protected void changeMaxValue(String NewValue_Arg) {
    if (!this.Ctrl.getUnit().equals("alpha")) {
      try {
        double NV = Double.parseDouble(NewValue_Arg);

        if (NV >= this.Ctrl.getMinimumValue()+Ctrl.getMinimumColoredSize()) {
          this.Ctrl.setMaximumValue(NV);
        }

        this.JComponent1.repaint();
      } catch (Exception Exception_Arg) {
        Exception_Arg.printStackTrace();
      }
    } else {
      double NV = NewValue_Arg.toUpperCase().charAt(0) - 'A';
      if (NV >= this.Ctrl.getMinimumValue()+Ctrl.getMinimumColoredSize()) {
        this.Ctrl.setMaximumValue(NV);
      }
    }
    this.Ctrl.requestFocus();
  } // changeMaxValue()


  /**
   * Description of the Method
   * 
   * @param MouseEvent_Arg
   *            Description of the Parameter
   */
  public void mouseClicked(MouseEvent MouseEvent_Arg) {
    final String Unit = this.Ctrl.getUnit();

    // Double click on the first number. The user want to change the minimum
    // value
    if (MouseEvent_Arg.getClickCount() > 1 && this.RectFirstLabel.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {
      this.Dragging = BiSliderPresentation.NONE;
      String Text = "" + this.Ctrl.getMinimumValue();
      if (Unit.equals("alpha")) {
        Text = "" + (char) ((int) this.Ctrl.getMinimumValue() + 'A');
      } else if (this.Ctrl.getMinimumValue() == (int) this.Ctrl.getMinimumValue()) {
        Text = "" + this.Ctrl.getMinimumValue();
      }

      this.JTextFieldMin.setText(Text);
      this.JTextFieldMin.setLocation(BiSliderPresentation.MARGIN_RULER_LEFT, 0);
      this.JTextFieldMin.setSize(this.JTextFieldMin.getPreferredSize().width + 10, this.JTextFieldMin.getPreferredSize().height);
      this.JTextFieldMin.setVisible(true);
      JComponent1.remove(this.JTextFieldMin);
      JComponent1.add(this.JTextFieldMin);
      this.JTextFieldMin.requestFocus();

      this.JTextFieldMin.addActionListener(new ActionListener() {

        public void actionPerformed(ActionEvent e) {
          while (BiSliderPresentation.this.JTextFieldMin.getActionListeners().length > 0) {
            BiSliderPresentation.this.JTextFieldMin.removeActionListener(BiSliderPresentation.this.JTextFieldMin.getActionListeners()[0]);
          }
          /*
           * if (Unit.equals("alpha")){ if
           * (JTextFieldMin.getText().length()==1 &&
           * JTextFieldMin.getText().charAt(0)>='A' &&
           * JTextFieldMin.getText().charAt(0)<='Z')
           * changeMinValue(""+(JTextFieldMin.getText().charAt(0)-'A'));
           * else if (JTextFieldMin.getText().length()==1 &&
           * JTextFieldMin.getText().charAt(0)>='a' &&
           * JTextFieldMin.getText().charAt(0)='z')
           * changeMinValue(""+(JTextFieldMin.getText().charAt(0)-'a')); }
           * else
           */
          BiSliderPresentation.this.changeMinValue(BiSliderPresentation.this.JTextFieldMin.getText());

          BiSliderPresentation.this.JTextFieldMin.setVisible(false);
        }
      });

      this.JComponent1.repaint();
    } // Double click on the last number. The user want to change the
    // maximum value
    else if (MouseEvent_Arg.getClickCount() > 1 && this.RectLastLabel.contains(MouseEvent_Arg.getX(),
            MouseEvent_Arg.getY())) {

      this.Dragging = BiSliderPresentation.NONE;
      String Text = "" + this.Ctrl.getMaximumValue();
      if (Unit.equals("alpha")) {
        Text = "" + (char) ((int) this.Ctrl.getMaximumValue() + 'A');
      } else if (this.Ctrl.getMaximumValue() == (int) this.Ctrl.getMaximumValue()) {
        Text = "" + (int) this.Ctrl.getMaximumValue();
      }

      this.JTextFieldMax.setText(Text);
      this.JTextFieldMax.setLocation(this.JComponent1.getWidth() - this.JTextFieldMax.getPreferredSize().width - BiSliderPresentation.MARGIN_RULER_RIGHT - 10, 0);
      this.JTextFieldMax.setSize(
              this.JTextFieldMax.getPreferredSize().width + 10,
              this.JTextFieldMax.getPreferredSize().height);
      this.JTextFieldMax.setVisible(true);
      this.JTextFieldMax.requestFocus();
      this.JTextFieldMax.addActionListener(new ActionListener() {

        public void actionPerformed(ActionEvent e) {
          while (BiSliderPresentation.this.JTextFieldMax.getActionListeners().length > 0) {
            BiSliderPresentation.this.JTextFieldMax.removeActionListener(BiSliderPresentation.this.JTextFieldMax.getActionListeners()[0]);
          }
          BiSliderPresentation.this.changeMaxValue(BiSliderPresentation.this.JTextFieldMax.getText());
          BiSliderPresentation.this.JTextFieldMax.setVisible(false);
        }
      });
    } else {
      return;
    }
    MouseEvent_Arg.consume();
  } // mouseClicked()

  /**
   * Description of the Method
   * 
   * @param MouseEvent_Arg
   *            Description of the Parameter
   */
  public void mouseEntered(MouseEvent MouseEvent_Arg) {
  }

  /**
   * Description of the Method
   * 
   * @param MouseEvent_Arg
   *            Description of the Parameter
   */
  public void mouseExited(MouseEvent MouseEvent_Arg) {
    this.MouseUnder = this.NOTHING;
    this.JComponent1.repaint();
  }
}
