/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.visutools.nav.bislider;

import java.util.ArrayList;

/**
 *
 * @author fredericvernier
 */
public class SegmentSupport implements java.io.Serializable {
  private  ArrayList<SegmentListener>  segmentListeners  = new ArrayList<SegmentListener>();

  /**
   * Adds a feature to the BiSliderListener attribute of the BiSliderSupport object
   *
   * @param l  The feature to be added to the BiSliderListener attribute
   */
  public synchronized void addSegmentListener(SegmentListener l) {
    // add a listener if it is not already registered
    if (!segmentListeners.contains(l))
      segmentListeners.add(l);
  }// addBiSliderListener()


  /**
   * Description of the Method
   *
   * @param l  Description of the Parameter
   */
  public synchronized void removeSegmentListener(SegmentListener l) {
    // remove it if it is registered
    if (segmentListeners.contains(l))
      segmentListeners.remove(l);
  }// removeBiSliderListener()


  /**
   * fire a new grab event
   *
   * @param Source_Arg      Source of the event.
   * @param ColorArray_Arg  Array describing the color correspondances.
   */
  void fireNotifyGrabSegment(BiSlider source, double Min, double Max, int d1, int GraduationCount){
    // Make a copy of the listener object ArrayList so that
    // it cannot be changed while we are firing events.
    ArrayList<SegmentListener>  v;
    synchronized (this) {
      v = (ArrayList<SegmentListener>)segmentListeners.clone();
    }

    // Fire the event to all listeners.
    int     count  = v.size();
    for (int i = 0; i < count; i++) {
      SegmentListener  listener  = (SegmentListener)v.get(i);
      listener.segmentGrabbed(new SegmentEvent(source, Min, Max, d1, GraduationCount));
    }
  }// fireNotifyGrabSegment()


  void fireNotifyDragSegment(BiSlider source, double Min, double Max, int d1, int GraduationCount){
    // Make a copy of the listener object ArrayList so that
    // it cannot be changed while we are firing events.
    ArrayList<SegmentListener>  v;
    synchronized (this) {
      v = (ArrayList<SegmentListener>)segmentListeners.clone();
    }

    // Fire the event to all listeners.
    int     count  = v.size();
    for (int i = 0; i < count; i++) {
      SegmentListener  listener  = (SegmentListener)v.get(i);
      listener.segmentDragged(new SegmentEvent(source, Min, Max, d1, GraduationCount));
    }
  }


  void fireNotifyReleaseSegment(BiSlider source, double Min, double Max, int d1, int GraduationCount){
    // Make a copy of the listener object ArrayList so that
    // it cannot be changed while we are firing events.
    ArrayList<SegmentListener>  v;
    synchronized (this) {
      v = (ArrayList<SegmentListener>)segmentListeners.clone();
    }

    // Fire the event to all listeners.
    int     count  = v.size();
    for (int i = 0; i < count; i++) {
      SegmentListener  listener  = (SegmentListener)v.get(i);
      listener.segmentReleased(new SegmentEvent(source, Min, Max, d1, GraduationCount));
    }
  }

}
