package com.visutools.nav.bislider;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.awt.image.WritableRaster;
import javax.swing.*;
import java.io.*;
import java.text.DecimalFormat;
import java.util.*;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.LineBorder;
import javax.swing.plaf.BorderUIResource;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.basic.BasicBorders;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;
import javax.swing.plaf.metal.OceanTheme;
/**
 * 5 examples of BiSlider in a JFrame to see the widget at work.
 * <br><br>
 * <table border=1 width = "90%">
 *   <tr>
 *     <td>
 *       Copyright 1997-2011 Frederic Vernier. All Rights Reserved.<br>
 *       <br>
 *       Permission to use, copy, modify and distribute this software and its documentation for educational, research and
 *       non-profit purposes, without fee, and without a written agreement is hereby granted, provided that the above copyright
 *       notice and the following three paragraphs appear in all copies.<br>
 *       <br>
 *       To request Permission to incorporate this software into commercial products contact
 *       Frederic Vernier, 19 butte aux cailles street, Paris, 75013, France. Tel: (+33) 871 747 387.
 *       eMail: Frederic.Vernier@laposte.net / Web site: http://vernier.frederic.free.fr
 *       <br>
 *       IN NO EVENT SHALL FREDERIC VERNIER BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 *       DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF FREDERIC
 *       VERNIER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.<br>
 *       <br>
 *       FREDERIC VERNIER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *       MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HERE UNDER IS ON AN "AS IS" BASIS, AND
 *       FREDERIC VERNIER HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.<br>
 *     </td>
 *   </tr>
 * </table>
 * <br>
 * <b>Project related :</b>  FiCell, FieldExplorer<br>
 * <br>
 * <b>Dates:</b><br>
 *   <li>Creation    : 1997<br>
 *   <li>Format      : 15/02/2004<br>
 *   <li>Last Modif  : 19/06/2005 <br>
 *<br>
 * <b>Bugs:</b><br>
 * <li><br>
 *<br>
 * <b>To Do:</b><br>
 *  <li><br>
 *<br>
 * @author Frederic Vernier, Frederic.Vernier@laposte.net
 * @version 1.5.2
 **/

public class Test {
  // all the properties of the application (parameters)
  protected static          Properties                                    BiSliderProperties              = new Properties();
  protected static          float                                         LineX                           = 0f;
  protected static          BufferedImage                                 bim                             = null;
  protected static MetalTheme [] metalThemes = {
    new OceanTheme(),
    new DefaultMetalTheme()
  };

  protected static double offset = 0;
  protected static double oldoffset = 0;
  protected static double oldprop = 0;


  public static void main(String[] Args) {
    try {
      File PropFileName = new File("properties/properties_BiSlider");
      System.out.println("Using property file :"+PropFileName);

      if (PropFileName.exists()){
        FileInputStream FileInputStream1 = new FileInputStream(PropFileName);
        BiSliderProperties.load(FileInputStream1);
        FileInputStream1.close();
      } else {
        BiSliderProperties.setProperty("Application.Name", "BiSlider");
        BiSliderProperties.setProperty("Application.Version", "1.0");
        BiSliderProperties.setProperty("Application.Build", "0001");
        FileOutputStream FileOutputStream1 = new FileOutputStream(PropFileName);
        BiSliderProperties.store(FileOutputStream1, "Values of the parameters of the application");
        FileOutputStream1.close();
      }
    } catch(IOException IOException_Arg){
      //IOException_Arg.printStackTrace();
    }


    final JFrame JFrame1 = new JFrame(" Examples of "+BiSliderProperties.getProperty("Application.Name")+
      " v"+BiSliderProperties.getProperty("Application.Version")+
      " build"+BiSliderProperties.getProperty("Application.Build"));
    JFrame1.setIconImage(new BiSliderBeanInfo().getIcon(BiSliderBeanInfo.ICON_COLOR_32x32));

    JFrame1.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    JFrame1.addWindowListener(new WindowAdapter() {
      @Override 
      public void windowClosing(WindowEvent evt) {
        JFrame1.setVisible(false);
        JFrame1.dispose();
        System.exit(0);
      }
    });

    final BiSlider BiSlider1h = new BiSlider(BiSlider.HSB);
    BiSlider1h.setVisible(true);
    BiSlider1h.setMinimumColor(new Color(Color.HSBtoRGB(0.01f, 1.0f, 1.0f)));
    BiSlider1h.setMaximumColor(new Color(Color.HSBtoRGB(0.99f, 1.0f, 1.0f)));
    BiSlider1h.setUnit("");
    BiSlider1h.setPrecise(true);
    BiSlider1h.setValues(0, 255);    
    BiSlider1h.setSegmentSize(16);
    BiSlider1h.setColoredValues(0, 255);

    final JPopupMenu JPopupMenu1h = BiSlider1h.createPopupMenu();
    BiSlider1h.addMouseListener(new MouseAdapter(){
      @Override 
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu1h.show(BiSlider1h, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });

    final BiSlider BiSlider1s = new BiSlider(BiSlider.HSB);
    BiSlider1s.setVisible(true);
    BiSlider1s.setMinimumColor(Color.GRAY);
    BiSlider1s.setMaximumColor(Color.RED);
    BiSlider1s.setUnit("");
    BiSlider1s.setPrecise(true);
    BiSlider1s.setValues(0, 255);    
    BiSlider1s.setSegmentSize(16);
    BiSlider1s.setColoredValues(0, 255);


    final JPopupMenu JPopupMenu1s = BiSlider1s.createPopupMenu();
    BiSlider1s.addMouseListener(new MouseAdapter(){
      @Override 
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu1s.show(BiSlider1s, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });

    final BiSlider BiSlider1b = new BiSlider(BiSlider.HSB);
    BiSlider1b.setVisible(true);
    BiSlider1b.setMinimumColor(Color.BLACK);
    BiSlider1b.setMaximumColor(Color.WHITE);
    BiSlider1b.setUnit("");
    BiSlider1b.setPrecise(true);
    BiSlider1b.setValues(0, 255);    
    BiSlider1b.setSegmentSize(16);
    BiSlider1b.setColoredValues(0, 255);

    final JPopupMenu JPopupMenu1b = BiSlider1b.createPopupMenu();
    BiSlider1b.addMouseListener(new MouseAdapter(){
      @Override 
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu1b.show(BiSlider1b, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });
    
    BiSlider1h.addBiSliderListener(new BiSliderAdapter(){
      /** something changed that modified the color gradient between min and max */
      @Override
      public void newColors(BiSliderEvent event) {
        BiSlider1s.repaint();
        BiSlider1b.repaint();
      }
    });
    BiSlider1s.addBiSliderListener(new BiSliderAdapter(){
      /** something changed that modified the color gradient between min and max */
      @Override
      public void newColors(BiSliderEvent event) {
        BiSlider1h.repaint();
        BiSlider1b.repaint();
      }
    });
    BiSlider1b.addBiSliderListener(new BiSliderAdapter(){
      /** something changed that modified the color gradient between min and max */
      @Override
      public void newColors(BiSliderEvent event) {
        BiSlider1h.repaint();
        BiSlider1s.repaint();
      }
    });

    BiSlider1h.addContentPainterListener(new ContentPainterListener(){
      public void paint(ContentPainterEvent event){
        Graphics2D Graphics2 = (Graphics2D)event.getGraphics();
        Rectangle Rect1 = event.getRectangle();
        Rectangle Rect2 = event.getBoundingRectangle();

        Shape oldClip = Graphics2.getClip();
        Graphics2.setClip(null);
        Graphics2.setColor(Color.WHITE);
        Graphics2.draw(Rect2);
        Graphics2.setClip(oldClip);
        //nothing to paint anyway
        if (Rect1.getWidth()<=0 || Rect1.getHeight()<=0)
          return;
        // try to allocate only when necessary the buffer
        if (bim==null || bim.getWidth()!=Rect2.getWidth() || bim.getHeight()!=Rect2.getHeight())
          bim = new BufferedImage(Rect2.width, Rect2.height, BufferedImage.TYPE_INT_RGB);
        
        if (event.getColor()!=null) {
          WritableRaster raster = bim.getRaster();

          float[] hsbval = new float[3];
          //float[] rgbval = new float[3];
          Color.RGBtoHSB(event.getColor().getRed(), event.getColor().getGreen(), event.getColor().getBlue(), hsbval);
          int[] pixels = ( (DataBufferInt) raster.getDataBuffer()).getData();

          float rs = (float)((BiSlider1s.getMaximumColoredValue()-BiSlider1s.getMinimumColoredValue())/255f);
          float ds = (float)(BiSlider1s.getMinimumColoredValue())/255f;
          float rb = (float)((BiSlider1b.getMaximumColoredValue()-BiSlider1b.getMinimumColoredValue())/255f);
          float db = (float)(BiSlider1b.getMinimumColoredValue())/255f;
          for (int y=0; y<bim.getHeight(); y++)
            for (int x=0; x<bim.getWidth(); x++) {
              int rgb = Color.HSBtoRGB(hsbval[0], ds+rs*((float)x)/bim.getWidth(), db+rb*((float)y)/bim.getHeight());
              pixels[y*bim.getWidth()+x] = rgb;          
            }

          Graphics2.drawImage(bim, Rect1.x, Rect1.y, null);
        }
      }
    });

    BiSlider1s.addContentPainterListener(new ContentPainterListener(){
      public void paint(ContentPainterEvent event){
        Graphics2D Graphics2 = (Graphics2D)event.getGraphics();
        Rectangle Rect1 = event.getRectangle();
        Rectangle Rect2 = event.getBoundingRectangle();
        Shape oldClip = Graphics2.getClip();
        Graphics2.setClip(null);
        Graphics2.setColor(Color.WHITE);
        Graphics2.draw(Rect2);
        Graphics2.setClip(oldClip);
        
        //nothing to paint anyway
        if (Rect1.getWidth()<=0 || Rect1.getHeight()<=0)
          return;
        // try to allocate only when necessary the buffer
        if (bim==null || bim.getWidth()!=Rect1.getWidth() || bim.getHeight()!=Rect1.getHeight())
          bim = new BufferedImage(Rect1.width, Rect1.height, BufferedImage.TYPE_INT_RGB);
        
        if (event.getColor()!=null) {
          WritableRaster raster = bim.getRaster();

          float[] hsbval = new float[3];
          //float[] rgbval = new float[3];
          Color.RGBtoHSB(event.getColor().getRed(), event.getColor().getGreen(), event.getColor().getBlue(), hsbval);
          int[] pixels = ( (DataBufferInt) raster.getDataBuffer()).getData();

          float rh = (float)((BiSlider1h.getMaximumColoredValue()-BiSlider1h.getMinimumColoredValue())/255f);
          float dh = (float)(BiSlider1h.getMinimumColoredValue())/255f;
          float rb = (float)((BiSlider1b.getMaximumColoredValue()-BiSlider1b.getMinimumColoredValue())/255f);
          float db = (float)(BiSlider1b.getMinimumColoredValue())/255f;
          for (int y=0; y<bim.getHeight(); y++)
            for (int x=0; x<bim.getWidth(); x++) {
              int rgb = Color.HSBtoRGB(dh+rh*((float)x)/bim.getWidth(), hsbval[1], db+rb*((float)y)/bim.getHeight());
              pixels[y*bim.getWidth()+x] = rgb;          
            }

          Graphics2.drawImage(bim, Rect1.x, Rect1.y, null);
        }
      }
    });


    BiSlider1b.addContentPainterListener(new ContentPainterListener(){
      public void paint(ContentPainterEvent event){
        Graphics2D Graphics2 = (Graphics2D)event.getGraphics();
        Rectangle Rect1 = event.getRectangle();
        Rectangle Rect2 = event.getBoundingRectangle();
        Shape oldClip = Graphics2.getClip();
        Graphics2.setClip(null);
        Graphics2.setColor(Color.WHITE);
        Graphics2.draw(Rect2);
        Graphics2.setClip(oldClip);
        
        //nothing to paint anyway
        if (Rect1.getWidth()<=0 || Rect1.getHeight()<=0)
          return;
        // try to allocate only when necessary the buffer
        if (bim==null || bim.getWidth()!=Rect1.getWidth() || bim.getHeight()!=Rect1.getHeight())
          bim = new BufferedImage(Rect1.width, Rect1.height, BufferedImage.TYPE_INT_RGB);
        
        if (event.getColor()!=null) {
          WritableRaster raster = bim.getRaster();

          float[] hsbval = new float[3];
          //float[] rgbval = new float[3];
          Color.RGBtoHSB(event.getColor().getRed(), event.getColor().getGreen(), event.getColor().getBlue(), hsbval);
          int[] pixels = ( (DataBufferInt) raster.getDataBuffer()).getData();

          float rh = (float)((BiSlider1h.getMaximumColoredValue()-BiSlider1h.getMinimumColoredValue())/255f);
          float dh = (float)(BiSlider1h.getMinimumColoredValue())/255f;
          float rs = (float)((BiSlider1s.getMaximumColoredValue()-BiSlider1s.getMinimumColoredValue())/255f);
          float ds = (float)(BiSlider1s.getMinimumColoredValue())/255f;
          for (int y=0; y<bim.getHeight(); y++)
            for (int x=0; x<bim.getWidth(); x++) {
              int rgb = Color.HSBtoRGB(dh+rh*((float)x)/bim.getWidth(), ds+rs*((float)y)/bim.getHeight(), hsbval[2]);
              pixels[y*bim.getWidth()+x] = rgb;          
            }

          Graphics2.drawImage(bim, Rect1.x, Rect1.y, null);
        }
      }
    });

    /*BiSlider1.addMouseListener(new MouseAdapter(){
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (!MouseEvent_Arg.isConsumed() && MouseEvent_Arg.getButton()==MouseEvent.BUTTON1 && MouseEvent_Arg.getClickCount()==2){
          Color Color1 = JColorChooser.showDialog(BiSlider1, "Choose a color for minimum values", BiSlider1.getMinimumColor());
          if (Color1!=null)
            BiSlider1.setMinimumColor(Color1);
          BiSlider1.repaint();
        }
      }
    });*/

    final BiSlider biSlider2 = new BiSlider(BiSlider.HSB);
    biSlider2.setToolTipText("No line to highlight the ticks here");
    biSlider2.setMinimumValue(-5);
    biSlider2.setMaximumValue(5);
    biSlider2.setSegmentSize(1);
    biSlider2.setMinimumColor(new Color(Color.HSBtoRGB(0.0f, 1.0f, 1.0f)));
    biSlider2.setMaximumColor(new Color(Color.HSBtoRGB(0.5f, 1.0f, 1.0f)));
    biSlider2.setColoredValues(-5, 5);
    biSlider2.setUnit("%");
    biSlider2.setBackground(Color.GRAY);
    biSlider2.setOpaque(true);
    biSlider2.setSliderBackground(new Color(152, 152, 192));
    biSlider2.setForeground(Color.WHITE);
    biSlider2.setSound(true);
    biSlider2.setArcSize(14);
    biSlider2.setFont(new Font("SansSerif", Font.ITALIC|Font.BOLD, 12));
    biSlider2.setPrecise(true);
    biSlider2.addContentPainterListener(new ContentPainterListener(){
      public void paint(ContentPainterEvent event){
        Graphics2D Graphics2 = (Graphics2D)event.getGraphics();
        Rectangle Rect1 = event.getRectangle();
        Rectangle Rect2 = event.getBoundingRectangle();
        if (event.getColor()!=null) {
          Graphics2.setColor(event.getColor());
          Graphics2.setPaint(new GradientPaint(Rect1.x, Rect1.y, event.getColor(),
            Rect1.x+(int)(LineX*Rect1.width), Rect1.y, event.getColor().brighter()));
          Graphics2.fillRect(Rect1.x, Rect1.y, (int)(LineX*Rect1.width), Rect1.height);

          Graphics2.setPaint(new GradientPaint(Rect1.x+(int)(LineX*Rect1.width), Rect1.y, event.getColor().brighter(),
            Rect1.x+Rect1.width, Rect1.y, event.getColor()));
          Graphics2.fillRect(Rect1.x+(int)(LineX*Rect1.width), Rect1.y, Rect1.width-(int)(LineX*Rect1.width), Rect1.height);
        }
      }
    });
    
    final BiSlider BiSlider2b = new BiSlider(BiSlider.H2SB);
    BiSlider2b.setToolTipText("No line to highlight the ticks here");
    BiSlider2b.setMinimumValue(-5);
    BiSlider2b.setMaximumValue(5);
    BiSlider2b.setSegmentSize(1);
    BiSlider2b.setMinimumColor(new Color(Color.HSBtoRGB(0.0f, 1.0f, 1.0f)));
    BiSlider2b.setMaximumColor(new Color(Color.HSBtoRGB(0.5f, 1.0f, 1.0f)));
    BiSlider2b.setColoredValues(-5, 5);
    BiSlider2b.setUnit("%");
    BiSlider2b.setBackground(Color.GRAY);
    BiSlider2b.setOpaque(true);
    BiSlider2b.setSliderBackground(new Color(152, 152, 192));
    BiSlider2b.setForeground(Color.WHITE);
    BiSlider2b.setSound(true);
    BiSlider2b.setArcSize(14);
    BiSlider2b.setFont(new Font("SansSerif", Font.ITALIC|Font.BOLD, 12));
    BiSlider2b.setPrecise(true);
    BiSlider2b.addContentPainterListener(new ContentPainterListener(){
      public void paint(ContentPainterEvent event){
        Graphics2D Graphics2 = (Graphics2D)event.getGraphics();
        Rectangle Rect1 = event.getRectangle();
        Rectangle Rect2 = event.getBoundingRectangle();
        if (event.getColor()!=null) {
          Graphics2.setColor(event.getColor());
          Graphics2.setPaint(new GradientPaint(Rect1.x, Rect1.y, event.getColor(),
            Rect1.x+(int)((1-LineX)*Rect1.width), Rect1.y, event.getColor().brighter()));
          Graphics2.fillRect(Rect1.x, Rect1.y, (int)((1-LineX)*Rect1.width), Rect1.height);

          Graphics2.setPaint(new GradientPaint(Rect1.x+(int)((1-LineX)*Rect1.width), Rect1.y, event.getColor().brighter(),
            Rect1.x+Rect1.width, Rect1.y, event.getColor()));
          Graphics2.fillRect(Rect1.x+(int)((1-LineX)*Rect1.width), Rect1.y, Rect1.width-(int)((1-LineX)*Rect1.width), Rect1.height);
        }
      }
    });
    
    Thread Thread1 = new Thread(){
      @Override 
      public void run() {
        while (true){
          while (LineX<1){
            LineX += 0.01f;
            try {sleep(20);} catch(InterruptedException  InterruptedException_Arg){}
            biSlider2.repaint();
            BiSlider2b.repaint();
            yield();
          }
          while (LineX>0) {
            LineX -= 0.01f;
            try {sleep(20);} catch(InterruptedException  InterruptedException_Arg){}
            biSlider2.repaint();
            BiSlider2b.repaint();
            yield();
          }
        }
      }
    };
    Thread1.setPriority(Thread.MIN_PRIORITY);
    Thread1.start();

    final JPopupMenu JPopupMenu2b = BiSlider2b.createPopupMenu();
    BiSlider2b.addMouseListener(new MouseAdapter(){
      @Override 
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu2b.show(BiSlider2b, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });
    final JPopupMenu JPopupMenu2 = biSlider2.createPopupMenu();
    biSlider2.addMouseListener(new MouseAdapter(){
      @Override 
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu2.show(biSlider2, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });

    final BiSlider BiSlider3 = new BiSlider(BiSlider.CENTRAL);
    BiSlider3.setToolTipText("Use a gradient of color to highlight the segments");
    BiSlider3.setUniformSegment(true);
    BiSlider3.setVisible(true);
    BiSlider3.setMinimumValue(0);
    BiSlider3.setMaximumValue(25);
    BiSlider3.setSegmentSize(1);
    BiSlider3.setMinimumColor(Color.YELLOW);
    BiSlider3.setMiddleColor(Color.BLACK);
    BiSlider3.setMaximumColor(Color.BLUE);
    BiSlider3.setColoredValues(10, 20);
    BiSlider3.setSound(true);
    BiSlider3.setUnit("alpha");
    BiSlider3.setIntSegments(true);
    BiSlider3.setIntValues(true);
    BiSlider3.setPrecise(false);
    BiSlider3.setArcSize(18);

    BiSlider3.addContentPainterListener(new ContentPainterListener(){
      public void paint(ContentPainterEvent event){
        Graphics2D Graphics2 = (Graphics2D)event.getGraphics();
        Rectangle Rect1 = event.getRectangle();
        Rectangle Rect2 = event.getBoundingRectangle();
        if (event.getColor()!=null) {
          Graphics2.setColor(event.getColor());
          Graphics2.setPaint(new GradientPaint(Rect2.x, Rect2.y, event.getColor().brighter(),
            Rect2.x+Rect2.width, Rect2.y+Rect2.height, event.getColor().darker()));
          Graphics2.fillRect(Rect1.x, Rect1.y, Rect1.width, Rect1.height);
        }
      }
    });

    final JPopupMenu JPopupMenu3 = BiSlider3.createPopupMenu();
    BiSlider3.addMouseListener(new MouseAdapter(){
      @Override
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu3.show(BiSlider3, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });


    // from an idea of laurent.sassi@free.fr
    final BiSlider BiSlider3b = new BiSlider(BiSlider.H2SB);
    BiSlider3b.setToolTipText("Use a gradient of color to highlight the segments");
    BiSlider3b.setUniformSegment(true);
    BiSlider3b.setVisible(true);
    BiSlider3b.setMinimumValue(0);
    BiSlider3b.setMaximumValue(25);
    BiSlider3b.setSegmentSize(1);
    BiSlider3b.setMinimumColor(Color.RED);
    BiSlider3b.setMiddleColor(Color.WHITE);
    BiSlider3b.setMaximumColor(Color.RED);
    BiSlider3b.setColoredValues(0, 1);
    BiSlider3b.setSound(true);
    BiSlider3b.setUnit("");
    BiSlider3b.setIntSegments(true);
    BiSlider3b.setIntValues(true);
    BiSlider3b.setPrecise(false);
    BiSlider3b.setArcSize(18);

    BiSlider3b.addContentPainterListener(new ContentPainterListener(){
      public void paint(ContentPainterEvent event){
        Graphics2D Graphics2 = (Graphics2D)event.getGraphics();
        Rectangle Rect1 = event.getRectangle();
        Rectangle Rect2 = event.getBoundingRectangle();
        if (event.getColor()!=null) {
          Graphics2.setColor(event.getColor());
          Graphics2.setPaint(new GradientPaint(Rect2.x, Rect2.y, event.getColor().brighter(),
            Rect2.x+Rect2.width, Rect2.y+Rect2.height, event.getColor().darker()));
          Graphics2.fillRect(Rect1.x, Rect1.y, Rect1.width, Rect1.height);
        } else {
          float hsbvals[] = new float[3];
          Color col = event.getColor2();
          Color.RGBtoHSB(col.getRed(), col.getGreen(), col.getBlue(), hsbvals);
          hsbvals[1] = 0.5f;
          hsbvals[2] = 0.8f;
          col = new Color(Color.HSBtoRGB(hsbvals[0], hsbvals[1], hsbvals[2]));
          Graphics2.setColor(col);
          Graphics2.fillRect(Rect1.x, Rect1.y, Rect1.width, Rect1.height);
        }
      }
    });
    BiSlider3b.addBiSliderListener(new BiSliderAdapter(){

      @Override
      public void prepareJumpValues(BiSliderEvent BiSliderEvent_Arg) {
        double center = (BiSliderEvent_Arg.getMinimumColored()+BiSliderEvent_Arg.getMaximumColored())/2;
        double prop   = 1-(center-BiSliderEvent_Arg.getMinimum())/(BiSliderEvent_Arg.getMaximum()-BiSliderEvent_Arg.getMinimum());
        oldprop = prop;
      }
      @Override
      public void notifyJumpValues(BiSliderEvent BiSliderEvent_Arg) {
        double center = (BiSliderEvent_Arg.getMinimumColored()+BiSliderEvent_Arg.getMaximumColored())/2;
        double prop   = 1-(center-BiSliderEvent_Arg.getMinimum())/(BiSliderEvent_Arg.getMaximum()-BiSliderEvent_Arg.getMinimum());
        offset += oldprop-prop;
        prop = (prop+offset)%1;
        Color col = new Color(Color.HSBtoRGB((float)prop, 1.0f, 1.0f));
        BiSlider3b.setMinimumColor(col);
        BiSlider3b.setMaximumColor(col);
      }

      @Override
      public void newValues(BiSliderEvent BiSliderEvent_Arg) {
        double center = (BiSliderEvent_Arg.getMinimumColored()+BiSliderEvent_Arg.getMaximumColored())/2;
        double prop   = 1-(center-BiSliderEvent_Arg.getMinimum())/(BiSliderEvent_Arg.getMaximum()-BiSliderEvent_Arg.getMinimum());
        prop = (prop+offset)%1;
        Color col = new Color(Color.HSBtoRGB((float)prop, 1.0f, 1.0f));

        BiSlider3b.setMinimumColor(col);
        BiSlider3b.setMaximumColor(col);
      }

    });

    BiSlider3b.addSegmentListener(new SegmentAdapter(){
      @Override
      public void segmentGrabbed(SegmentEvent event) {
        BiSlider slider = (BiSlider)event.getSource();
        oldprop = ((double)event.dx)/ (slider.getRulerWidth());
        //System.out.println("grab "+oldprop);
        oldoffset = offset;
      }
      @Override
      public void segmentDragged(SegmentEvent event) {
        BiSlider slider = (BiSlider)event.getSource();
        double prop = ((double)event.dx)/ (slider.getRulerWidth());
        
        offset = oldoffset-(prop-oldprop);
        offset = offset*(slider.getMaximumValue()-slider.getMinimumValue());
        offset = (int)offset;
        offset = offset/(slider.getMaximumValue()-slider.getMinimumValue());
        //System.out.println("  drag "+prop+" => offset="+offset);
        
        double center = (slider.getMinimumColoredValue()+slider.getMaximumColoredValue())/2;
        prop   = 1-(center-slider.getMinimumValue())/(slider.getMaximumValue()-slider.getMinimumValue());
        prop = (prop+offset)%1;
        Color col = new Color(Color.HSBtoRGB((float)prop, 1.0f, 1.0f));
        BiSlider3b.setMinimumColor(col);
        BiSlider3b.setMaximumColor(col);
        BiSlider3b.repaint();
      }
    });



    final JPopupMenu JPopupMenu3b = BiSlider3b.createPopupMenu();
    BiSlider3b.addMouseListener(new MouseAdapter(){
      @Override 
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu3b.show(BiSlider3b, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });



    final BiSlider BiSlider4 = new BiSlider(BiSlider.RGB, false);
    BiSlider4.setToolTipText("Like a thermometer");
    BiSlider4.setUniformSegment(true);
    BiSlider4.setVisible(true);
    BiSlider4.setMinimumValue(0);
    BiSlider4.setMaximumValue(100);
    BiSlider4.setSegmentSize(5);
    BiSlider4.setMinimumColor(Color.BLUE);
    BiSlider4.setMaximumColor(Color.RED);
    BiSlider4.setColoredValues(0, 100);
    BiSlider4.setUnit(""+(char)(0xB0)); // 167
    BiSlider4.setPrecise(true);
    BiSlider4.setArcSize(18);

    final JPopupMenu JPopupMenu4 = BiSlider4.createPopupMenu();
    BiSlider4.addMouseListener(new MouseAdapter(){
      @Override 
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu4.show(BiSlider4, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });

    final BiSlider BiSlider5 = new BiSlider(BiSlider.RGB, true);
    BiSlider5.setToolTipText("Like a sea-depth meter");
    BiSlider5.setUniformSegment(true);
    BiSlider5.setVisible(true);
    BiSlider5.setMinimumValue(0);
    BiSlider5.setMaximumValue(100);
    BiSlider5.setSegmentSize(10);
    BiSlider5.setMinimumColor(Color.BLUE);
    BiSlider5.setMaximumColor(Color.BLACK);
    BiSlider5.setColoredValues(20, 70);
    BiSlider5.setBackground(Color.BLACK);
    BiSlider5.setSliderBackground(new Color(96, 96, 156));
    BiSlider5.setForeground(Color.YELLOW);
    BiSlider5.setFont(new Font("Serif", Font.ITALIC|Font.BOLD, 12));
    BiSlider5.setUnit("m");
    BiSlider5.setPrecise(true);
    BiSlider5.setArcSize(10);

    final JPopupMenu JPopupMenu5 = BiSlider5.createPopupMenu();
    BiSlider5.addMouseListener(new MouseAdapter(){
      @Override
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu5.show(BiSlider5, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });


    final BiSlider biSlider6 = new BiSlider(BiSlider.CENTRAL);
    biSlider6.setToolTipText("<html><li>custom background is a histogramm<br><li>maximum selection size</html>");
    biSlider6.setUniformSegment(true);
    biSlider6.setVisible(true);
    biSlider6.setMinimumValue(0);
    biSlider6.setMaximumValue(100);
    biSlider6.setSegmentSize(4);
    biSlider6.setMinimumColor(Color.RED);
    biSlider6.setMiddleColor(Color.BLACK);
    biSlider6.setMaximumColor(Color.RED);
    biSlider6.setMaximumColoredSize(50);
    biSlider6.setColoredValues(25, 65);
    biSlider6.setUnit("$");
    biSlider6.setBackground(Color.BLACK);
    biSlider6.setPrecise(true);
    biSlider6.setArcSize(18);
    biSlider6.addContentPainterListener(new ContentPainterListener() {
      public void paint(ContentPainterEvent event){
        Graphics2D g2 = (Graphics2D)event.getGraphics();
        //Rectangle Rect1 = event.getRectangle();
        Rectangle rect2 = event.getBoundingRectangle();
        //double BarHeight = Math.abs(Math.cos(Math.PI*(rect2.x+rect2.width/2) / biSlider6.getWidth()));
        //BarHeight = (double)(rect2.x+rect2.width/2) / biSlider6.getWidth();
        //BarHeight = Math.random();
        float X = ((float)rect2.x-biSlider6.getWidth()/2)/biSlider6.getWidth()*6;
        double BarHeight = 1-Math.exp((-1*X*X)/2);
        X = ((float)(rect2.x-rect2.width)-biSlider6.getWidth()/2)/biSlider6.getWidth()*6;
        double BarHeight2 = 1-Math.exp((-1*X*X)/2);

        if (event.getColor()!=null) {
          g2.setColor(biSlider6.getSliderBackground());
          //g2.fillRect(rect2.x, rect2.y, rect2.width, (int)((BarHeight*rect2.height)));
          g2.setColor(event.getColor());
          g2.fillRect(rect2.x, rect2.y+(int)((BarHeight*rect2.height)), rect2.width+1, 1+(int)(((1-BarHeight)*rect2.height)));
          //g2.drawRect(rect2.x, rect2.y+(int)((BarHeight*rect2.height)), rect2.width+1, 1+(int)(((1-BarHeight)*rect2.height)));
        } else {
          g2.setColor(new Color(255, 255, 218, 64));
          g2.fillRect(rect2.x, rect2.y+(int)((BarHeight*rect2.height)), rect2.width+1, 1+(int)(((1-BarHeight)*rect2.height)));
        }
        g2.setColor(Color.BLACK);
        //g2.drawRect(rect2.x, rect2.y+(int)((BarHeight*rect2.height)), rect2.width-1, (int)(((1-BarHeight)*rect2.height)));
        g2.drawLine(rect2.x, rect2.y+(int)((BarHeight*rect2.height)), rect2.x+rect2.width-1, rect2.y+(int)((BarHeight*rect2.height)));
        g2.drawLine(rect2.x, rect2.y+(int)((Math.min(BarHeight, BarHeight2)*rect2.height)), rect2.x, rect2.y+rect2.height);
      }
    });

    final JPopupMenu JPopupMenu6 = biSlider6.createPopupMenu();
    biSlider6.addMouseListener(new MouseAdapter(){
      @Override
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          JPopupMenu6.show(biSlider6, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });

    final DecimalFormat  decimalFormater  = new DecimalFormat("00");
    final BiSlider biSlider7 = new BiSlider(BiSlider.RGB);
    biSlider7.setToolTipText("<html><li>Custom labels to display time<br><li>minimum selection size</html>");
    biSlider7.setUniformSegment(false);
    biSlider7.setVisible(true);
    biSlider7.setMinimumValue(0);
    biSlider7.setMaximumValue(30*60*60*1.5);
    biSlider7.setSegmentSize(30*60);
    biSlider7.setMinimumColor(Color.WHITE);
    biSlider7.setMaximumColor(Color.BLUE);
    biSlider7.setMinimumColoredSize(30*60*5);
    biSlider7.setColoredValues(0, 30*60*60*1.5); // one and half hour
    biSlider7.setPreferredSize(new Dimension(500, 36));
    biSlider7.setMarginRulerTop(19);
    biSlider7.setUnit("'");
    biSlider7.setOpaque(true);
    biSlider7.setBackground(Color.GRAY);
    biSlider7.setSliderBackground(new Color(152, 152, 192));
    biSlider7.setForeground(Color.WHITE);
    biSlider7.setPrecise(true);
    biSlider7.setPrecisionAmplitudeRatio(0.03f);
    biSlider7.addUnitPainterListener(new UnitPainterListener(){
      public void paintTick(UnitPainterEvent event){
        Graphics2D g2 = (Graphics2D)event.getGraphics();
        g2.setColor(biSlider7.getForeground());
        Rectangle rect2 = event.getRectangle();
        FontMetrics fontMetrics0 = g2.getFontMetrics();
        Font oldfont = g2.getFont();
        
        if (event.getValue()%(30*60*60)==0){
          Font newfont = g2.getFont().deriveFont(Font.ITALIC, g2.getFont().getSize()+2);
          g2.setFont(newfont);
          int val = (int)(event.getValue()/(30*60*60));
          String text = ""+val+"h";
          g2.drawString(text, rect2.x+rect2.width/2-fontMetrics0.stringWidth(text)/2, rect2.y+rect2.height-5);
        } else  if (event.getValue()%(30*60*5)==0){
          g2.setColor(biSlider7.getForeground().darker());
          Font newfont = g2.getFont().deriveFont(Font.ITALIC, g2.getFont().getSize()-2);
          g2.setFont(newfont);
          int hrs = (int)(event.getValue()/(30*60*60));
          int val = (int)((event.getValue()-hrs*60*60*30)/(30*60));
          String text = ""+decimalFormater.format(val)+"'";
          g2.drawString(text, rect2.x+rect2.width/2-fontMetrics0.stringWidth(text)/2, rect2.y+rect2.height-1);
        } else if (event.getSource()!=biSlider7 && event.getSegmentIndex()>=0){ //  not painting on bislider = painting a tick on precision popup
          Font newfont = g2.getFont().deriveFont(Font.PLAIN, g2.getFont().getSize()-1);
          g2.setFont(newfont);
          int hrs = (int)(event.getValue()/(30*60*60));
          int val = (int)((event.getValue()-hrs*60*60*30));
          int min = val/(30*60);
          val = val-min*30*60;
          int sec= val/30;
          String text = hrs+"h"+decimalFormater.format(min)+"'"+decimalFormater.format(sec)+"\"";
          g2.drawString(text, rect2.x+rect2.width/2-fontMetrics0.stringWidth(text)/2, rect2.y+rect2.height-1);
        } else if (event.getSource()!=biSlider7 && event.getSegmentIndex()<0){ //  not painting on bislider = painting the precise value on precision popup
          Font newfont = g2.getFont().deriveFont(Font.BOLD, g2.getFont().getSize()-1);
          g2.setFont(newfont);
          int hrs = (int)(event.getValue()/(30*60*60));
          int val = (int)((event.getValue()-hrs*60*60*30));
          int min = val/(30*60);
          val = val-min*30*60;
          int sec= val/30;
          int frm = val%30;
          String text = hrs+"h"+decimalFormater.format(min)+"'"+decimalFormater.format(sec)+"\"  F"+decimalFormater.format(frm);
          g2.drawString(text, rect2.x+rect2.width/2-fontMetrics0.stringWidth(text)/2, rect2.y+rect2.height-1);
        }
        g2.setFont(oldfont);
      }
      public void paintValue(UnitPainterEvent event){
        Graphics2D g2 = (Graphics2D)event.getGraphics();
        g2.setColor(biSlider7.getForeground());
        Rectangle rect2 = event.getRectangle();
        int val = (int)(event.getValue()/(30*60));
        //g2.setColor(Color.BLUE);
        //g2.fillRect(rect2.x, rect2.y, rect2.width, rect2.height);
        g2.setColor(biSlider7.getForeground());
        FontMetrics fontMetrics0 = g2.getFontMetrics();

        int hrs = (int)(event.getValue()/(30*60*60));
        int min = (int)((event.getValue()-hrs*60*60*30)/(30*60));
        int sec = (int)((event.getValue()-hrs*60*60*30-min*30*60)/(30));
        String text = ""+hrs+"h";
        if (min<10)
          text+="0"+min+"'";
        else
          text+=min+"'";
        if (sec<10)
          text+="0"+sec+"\"";
        else
          text+=sec+"\"";

        if (event.getValue()==biSlider7.getMinimumColoredValue())
          g2.drawString(text, rect2.x+rect2.width-fontMetrics0.stringWidth(text), rect2.y+rect2.height-2);
        else
          g2.drawString(text, rect2.x, rect2.y+rect2.height-2);
      }
    }); 
    biSlider7.setPrecise(true);
    biSlider7.setArcSize(8);

    final JPopupMenu jPopupMenu7 = biSlider7.createPopupMenu();
    biSlider7.addMouseListener(new MouseAdapter(){
      @Override
      public void mousePressed(MouseEvent MouseEvent_Arg){
        if (MouseEvent_Arg.getButton()==MouseEvent.BUTTON3){
          jPopupMenu7.show(biSlider7, MouseEvent_Arg.getX(), MouseEvent_Arg.getY());
        }
      }
    });

    JFrame1.getContentPane().setLayout(new BorderLayout(2, 2));
    JPanel JPanel1= new JPanel();
    JPanel JPanel2= new JPanel();
    JPanel1.setLayout(new BoxLayout(JPanel1, BoxLayout.Y_AXIS));
    JPanel2.setLayout(new BorderLayout(2, 2));

    final String initialText = "\n\n\n Use this BiSlider to see the events generated\n";
    JPanel2.add(biSlider7, BorderLayout.NORTH);
    JPanel2.add(biSlider6, BorderLayout.CENTER);
    final JTextArea JTextArea5 = new  JTextArea(initialText);
    JPanel2.add(JTextArea5, BorderLayout.SOUTH);
    biSlider6.addBiSliderListener(new BiSliderAdapter(){
      /** something changed that modified the color gradient between min and max */
      @Override
      public void newColors(BiSliderEvent BiSliderEvent_Arg) {
        String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"newColors \n";
        JTextArea5.setText(newText);      
      }
      /**  min or max values changed  */  
      @Override
      public void newValues(BiSliderEvent BiSliderEvent_Arg) {
        String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"newValues "+BiSliderEvent_Arg.getMinimum()+" <->"+BiSliderEvent_Arg.getMaximum()+"\n";
        JTextArea5.setText(newText);      
      }
      /**  min selected value changed  */
      @Override
      public void newMinValue(BiSliderEvent BiSliderEvent_Arg) {
        String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"newMinValue "+BiSliderEvent_Arg.getMinimum()+"\n";
        JTextArea5.setText(newText);         
      }
      /**  max selected value changed  */
      @Override
      public void newMaxValue(BiSliderEvent BiSliderEvent_Arg) {
        String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"newMaxValue "+BiSliderEvent_Arg.getMaximum()+"\n";
        JTextArea5.setText(newText);         
      }
      /**  selected segments changed  */
      @Override
      public void newSegments(BiSliderEvent BiSliderEvent_Arg) {
        String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"newSegments \n";
        JTextArea5.setText(newText);      
       }
      /**  min selected value changed  */
      @Override
      public void newMin(BiSliderEvent BiSliderEvent_Arg) {
        String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"newMin "+BiSliderEvent_Arg.getMinimum()+"\n";
        JTextArea5.setText(newText);         
      }
      /**  max selected value changed  */
      @Override
      public void newMax(BiSliderEvent BiSliderEvent_Arg) {
        String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"newMax "+BiSliderEvent_Arg.getMaximum()+"\n";
        JTextArea5.setText(newText);         
      }  
      /**  max selected value changed  */
      @Override
      public void newGlobalRange(BiSliderEvent BiSliderEvent_Arg) {
        String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"newglobalRange "+BiSliderEvent_Arg.getMinimum()+"-"+BiSliderEvent_Arg.getMaximum()+"\n";
        JTextArea5.setText(newText);         
      }
      /**  it's about to jump  */
      @Override
      public void prepareJumpValues(BiSliderEvent BiSliderEvent_Arg) {
         String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"prepareJumpValues "+BiSliderEvent_Arg.getMinimumColored()+"-"+BiSliderEvent_Arg.getMaximumColored()+"\n";
        JTextArea5.setText(newText);
      }

      /**  it has jumped  */
      @Override
      public void notifyJumpValues(BiSliderEvent BiSliderEvent_Arg) {
         String newText = JTextArea5.getText();
        newText = newText.substring(newText.indexOf("\n")+1)+"notifyJumpValues "+BiSliderEvent_Arg.getMinimumColored()+"-"+BiSliderEvent_Arg.getMaximumColored()+"\n";
        JTextArea5.setText(newText);
      }
    });
    
    JSplitPane JSplitPane2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, JPanel1, JPanel2);
    JSplitPane JSplitPane1 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, JSplitPane2, BiSlider5);

    JFrame1.getContentPane().add(BorderLayout.CENTER, JSplitPane1);
    JFrame1.getContentPane().add(BorderLayout.WEST, BiSlider4);

    JTextArea JTextArea1 = new  JTextArea(
      "* Ten Examples of BiSlider Bean by Frederic Vernier. Updated january 2009. Version 1.51\n"+
      "* This bean can be imported in a graphical java Interface Builder.\n"+
      "* Read the ReadMe.txt File for legal notice and javadoc for API.\n"+
      "* Right click to bring a popup menu with user's options.\n"+
      "* Double click the triangles to bring a popup slider to set values with precision since version 1.3.5."
      );
    JTextArea1.setEditable(false);
    JPanel1.add(JTextArea1);
    JPanel1.add(BiSlider1h);
    JPanel1.add(BiSlider1s);
    JPanel1.add(BiSlider1b);

    JTextArea JTextArea2 = new  JTextArea(
      "+ It now works for integer or double values since version 1.3.3\n"+
      "+ We force the gaps to be uniform in the 3rd, 4th and 5th exemples (= int division).\n"+
      "+ There are 4 different kinds of color interpolation: HSB, H2SB CENTRAL and RGB\n"+
      "+ If SegmentCount equals the range this bean behaves like a range slider.\n"+
      "+ Since v1.3.5 corners can be rounded and foreground color can be changed."
      );
    JTextArea2.setEditable(false);
    JPanel1.add(JTextArea2);
    JPanel1.add(biSlider2);
    JPanel1.add(BiSlider2b);
    
    JTextArea JTextArea3 = new  JTextArea(
      "- Double click a non-colored gap to select it as a the segment.\n"+
      "- Click and drag the maximum or the minimum triangle to change them.\n"+
      "- Shift+double click to extend the selection with a new gap.\n"+
      "- Shift click triangle or segment will align it on graduation.\n"+
      "- While dragging a triangle, turn 90� to open the precision popup without releasing the mouse button."
      );
    JTextArea3.setEditable(false);
    JPanel1.add(JTextArea3);
    JPanel1.add(BiSlider3);
    JPanel1.add(BiSlider3b);

    JTextArea JTextArea4 = new  JTextArea(
      "- Alt+click+drag a triangle to select a range around the central value.\n"+
      "- Drag&Drop the minimum value (text in bold) of the legend to set the SegmentCount.\n"+
      "- Shift Drag&Drop the minimum value (text) to stay on the int values.\n"+
      "- Double click the minimum or maximum value (text) to change the scope of the bean.\n"+
      "- Font painting is now anti-aliased and font can be changed since v1.3.5"
      );
    JTextArea4.setEditable(false);
    //JPanel1.add(JTextArea4);
    JSlider JSlider1 = new JSlider();
    JSlider1.setPaintLabels(true);
    JSlider1.setPaintTicks(true);
    JSlider1.setPaintTrack(true);
    JSlider1.setSnapToTicks(true);
    JSlider1.setMajorTickSpacing(10);
    JSlider1.setMinorTickSpacing(1);
    JPanel1.add(JSlider1);

    JTextArea4.setEditable(false);
    JPanel1.add(JTextArea4);

    //System.out.println("BiSlider1 = "+BiSlider1);
    JMenuBar JMenuBar1         = new JMenuBar();
    JMenu JMenuPLaF            = new JMenu("Look&Feel");
    ButtonGroup LnFButtonGroup = new ButtonGroup();

    JMenuPLaF.getAccessibleContext().setAccessibleDescription("This menu supports looks and feel selection");
    JMenuPLaF.setMnemonic(KeyEvent.VK_L);

    //boolean FirstMetal = true;
    class PLaFAction extends AbstractAction {
      private String LnFClassName;
      private String Title;

      public PLaFAction (String LnFClassName_Arg, String Title_Arg){
        super(Title_Arg);
        LnFClassName = LnFClassName_Arg;
        Title        = Title_Arg;
      }
      public void actionPerformed(ActionEvent ActionEvent_Arg) {
        try {
          if (Title.startsWith("Metal") && Title.endsWith("Steel")) {
            MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme());
          } else if (Title.startsWith("Metal") && Title.endsWith("Ocean")) {
            Class Class1   = Class.forName("javax.swing.plaf.metal.OceanTheme");
            Object Object1 = Class1.newInstance();
            MetalLookAndFeel.setCurrentTheme((MetalTheme)Object1);
          } else if (Title.startsWith("Metal") && Title.endsWith("Aqua")) {
            MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme(){
            @Override
              public String getName() { return "Aqua"; }
              private final ColorUIResource primary1 = new ColorUIResource(102, 153, 153);
              private final ColorUIResource primary2 = new ColorUIResource(128, 192, 192);
              private final ColorUIResource primary3 = new ColorUIResource(159, 235, 235);
            @Override
              protected ColorUIResource getPrimary1() { return primary1; }
             @Override
             protected ColorUIResource getPrimary2() { return primary2; }
            @Override
              protected ColorUIResource getPrimary3() { return primary3; }

          });
          } else if (Title.startsWith("Metal") && Title.endsWith("Charcoal")) {
            MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme(){
            @Override
            public String getName() { return "Charcoal"; }
            private final ColorUIResource primary1 = new ColorUIResource(66, 33, 66);
            private final ColorUIResource primary2 = new ColorUIResource(90, 86, 99);
            private final ColorUIResource primary3 = new ColorUIResource(99, 99, 99);

            private final ColorUIResource secondary1 = new ColorUIResource(0, 0, 0);
            private final ColorUIResource secondary2 = new ColorUIResource(51, 51, 51);
            private final ColorUIResource secondary3 = new ColorUIResource(102, 102, 102);

            private final ColorUIResource black = new ColorUIResource(222, 222, 222);
            private final ColorUIResource white = new ColorUIResource(0, 0, 0);

            @Override
            protected ColorUIResource getPrimary1() { return primary1; }
            @Override
            protected ColorUIResource getPrimary2() { return primary2; }
            @Override
            protected ColorUIResource getPrimary3() { return primary3; }

            @Override
            protected ColorUIResource getSecondary1() { return secondary1; }
            @Override
            protected ColorUIResource getSecondary2() { return secondary2; }
            @Override
            protected ColorUIResource getSecondary3() { return secondary3; }

            @Override
            protected ColorUIResource getBlack() { return black; }
            @Override
            protected ColorUIResource getWhite() { return white; }
          });
          } else if (Title.startsWith("Metal") && Title.endsWith("Contrast")) {
            MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme() {
            @Override
              public String getName() { return "Contrast"; }

              private final ColorUIResource primary1 = new ColorUIResource(0, 0, 0);
              private final ColorUIResource primary2 = new ColorUIResource(204, 204, 204);
              private final ColorUIResource primary3 = new ColorUIResource(255, 255, 255);
              private final ColorUIResource primaryHighlight = new ColorUIResource(102,102,102);

              private final ColorUIResource secondary2 = new ColorUIResource(204, 204, 204);
              private final ColorUIResource secondary3 = new ColorUIResource(255, 255, 255);
              private final ColorUIResource controlHighlight = new ColorUIResource(102,102,102);

            @Override
              protected ColorUIResource getPrimary1() { return primary1; }
            @Override
              protected ColorUIResource getPrimary2() { return primary2; }
            @Override
              protected ColorUIResource getPrimary3() { return primary3; }
            @Override
              public ColorUIResource getPrimaryControlHighlight() { return primaryHighlight;}

            @Override
              protected ColorUIResource getSecondary2() { return secondary2; }
            @Override
              protected ColorUIResource getSecondary3() { return secondary3; }
            @Override
              public ColorUIResource getControlHighlight() { return super.getSecondary3(); }

            @Override
              public ColorUIResource getFocusColor() { return getBlack(); }

            @Override
              public ColorUIResource getTextHighlightColor() { return getBlack(); }
            @Override
              public ColorUIResource getHighlightedTextColor() { return getWhite(); }

            @Override
              public ColorUIResource getMenuSelectedBackground() { return getBlack(); }
            @Override
              public ColorUIResource getMenuSelectedForeground() { return getWhite(); }
            @Override
              public ColorUIResource getAcceleratorForeground() { return getBlack(); }
            @Override
              public ColorUIResource getAcceleratorSelectedForeground() { return getWhite(); }
            @Override


              public void addCustomEntriesToTable(UIDefaults table) {
                  Border blackLineBorder = new BorderUIResource(new LineBorder( getBlack() ));
                  Object textBorder = new BorderUIResource( new CompoundBorder(
                                                                 blackLineBorder,
                                                                 new BasicBorders.MarginBorder()));
                  table.put( "ToolTip.border", blackLineBorder);
                  table.put( "TitledBorder.border", blackLineBorder);
                  table.put( "TextField.border", textBorder);
                  table.put( "PasswordField.border", textBorder);
                  table.put( "TextArea.border", textBorder);
                  table.put( "TextPane.border", textBorder);
                  table.put( "EditorPane.border", textBorder);
              }
            });
          } else if (Title.startsWith("Metal") && Title.endsWith("Ruby")) {
            MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme() {
              private final ColorUIResource primary1 = new ColorUIResource(80, 10, 22);
              private final ColorUIResource primary2 = new ColorUIResource(193, 10, 44);
              private final ColorUIResource primary3 = new ColorUIResource(244, 10, 66);

            @Override
              protected ColorUIResource getPrimary1() { return primary1; }
            @Override
              protected ColorUIResource getPrimary2() { return primary2; }
            @Override
              protected ColorUIResource getPrimary3() { return primary3; }
            });

          } else if (Title.startsWith("Metal") && Title.endsWith("Emerald")) {
            MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme() {
            @Override
              public String getName() { return "Emerald"; }

              private final ColorUIResource primary1 = new ColorUIResource(51, 142, 71);
              private final ColorUIResource primary2 = new ColorUIResource(102, 193, 122);
              private final ColorUIResource primary3 = new ColorUIResource(153, 244, 173);

            @Override
              protected ColorUIResource getPrimary1() { return primary1; }
            @Override
              protected ColorUIResource getPrimary2() { return primary2; }
            @Override
              protected ColorUIResource getPrimary3() { return primary3; }
            });
          }

          UIManager.setLookAndFeel(LnFClassName);
          SwingUtilities.updateComponentTreeUI(JFrame1);
          JFrame1.pack();
          Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
          JFrame1.setLocation((screenSize.width-JFrame1.getWidth())/2,(screenSize.height-JFrame1.getHeight())/2);

        } catch (Exception Exception_Arg) {
          System.out.println("Failed loading L&F: " + LnFClassName);
          Exception_Arg.printStackTrace();
        }
      }
    }

    boolean FirstMetal = true;
    UIManager.LookAndFeelInfo[]  lafis  = UIManager.getInstalledLookAndFeels();
    //for (int i = 0; i < lafis.length; i++)
      //System.out.println("Plaf: " + lafis[i].getClassName());
    if (lafis != null)
      for (int i = 0; i < lafis.length; i++)
        try {
          final LookAndFeel  currLAF  = (LookAndFeel)Class.forName(lafis[i].getClassName()).newInstance();
          // System.out.println("Plaf: " + lafis[i].getClassName());
          if (currLAF.isSupportedLookAndFeel()) {
            JRadioButtonMenuItem  jMenuItemLnF  = new JRadioButtonMenuItem(lafis[i].getName());
            final  String     LnFClassName  = lafis[i].getClassName();
            LnFButtonGroup.add(jMenuItemLnF);

            if (currLAF instanceof MetalLookAndFeel && FirstMetal && System.getProperty("java.version").startsWith("1.5")) {
              jMenuItemLnF.setSelected(true);
              jMenuItemLnF.setText("Metal Ocean");
              jMenuItemLnF.setAction(new PLaFAction(LnFClassName, jMenuItemLnF.getText()));
               FirstMetal = false;
              i--;
              JMenuPLaF.add(jMenuItemLnF);
            }
            else if (currLAF instanceof MetalLookAndFeel && !FirstMetal) {
              jMenuItemLnF.setText("Metal Steel");
              jMenuItemLnF.setAction(new PLaFAction(LnFClassName, jMenuItemLnF.getText()));
              JMenuPLaF.add(jMenuItemLnF);

              jMenuItemLnF  = new JRadioButtonMenuItem("Metal Aqua");
              LnFButtonGroup.add(jMenuItemLnF);
              jMenuItemLnF.setAction(new PLaFAction(LnFClassName, jMenuItemLnF.getText()));
              JMenuPLaF.add(jMenuItemLnF);

              jMenuItemLnF  = new JRadioButtonMenuItem("Metal Charcoal");
              LnFButtonGroup.add(jMenuItemLnF);
              jMenuItemLnF.setAction(new PLaFAction(LnFClassName, jMenuItemLnF.getText()));
              JMenuPLaF.add(jMenuItemLnF);

              jMenuItemLnF  = new JRadioButtonMenuItem("Metal High Contrast");
              LnFButtonGroup.add(jMenuItemLnF);
              jMenuItemLnF.setAction(new PLaFAction(LnFClassName, jMenuItemLnF.getText()));
              JMenuPLaF.add(jMenuItemLnF);

              jMenuItemLnF  = new JRadioButtonMenuItem("Metal Ruby");
              LnFButtonGroup.add(jMenuItemLnF);
              jMenuItemLnF.setAction(new PLaFAction(LnFClassName, jMenuItemLnF.getText()));
              JMenuPLaF.add(jMenuItemLnF);

              jMenuItemLnF  = new JRadioButtonMenuItem("Metal Emerald");
              LnFButtonGroup.add(jMenuItemLnF);
              jMenuItemLnF.setAction(new PLaFAction(LnFClassName, jMenuItemLnF.getText()));
              JMenuPLaF.add(jMenuItemLnF);
            } else{
              jMenuItemLnF.setAction(new PLaFAction(LnFClassName, jMenuItemLnF.getText()));
              JMenuPLaF.add(jMenuItemLnF);
            }

          }
        } catch (Exception Exception_Arg) {
          System.out.println("Failed loading L&F list ");
          Exception_Arg.printStackTrace();
        }

    JMenuBar1.add(JMenuPLaF);
    JFrame1.setJMenuBar(JMenuBar1);

    //System.out.println("BiSlider1 = "+BiSlider1);
    JFrame1.pack();
    JFrame1.setSize(JFrame1.getWidth(), JFrame1.getHeight()+50);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    JFrame1.setLocation((screenSize.width-JFrame1.getWidth())/2,(screenSize.height-JFrame1.getHeight())/2);

    JFrame1.setVisible(true);
  }// main()

}