package com.visutools.nav.bislider;

import java.awt.*;

/**
 * The event a BiSlider can generate to ask a refresh of the units <br>
 * <br>
 *
 * <table border=1 width = "90%">
 *   <tr>
 *     <td>
 *       Copyright 1997-2011 Frederic Vernier. All Rights Reserved.<br>
 *       <br>
 *       Permission to use, copy, modify and distribute this software and its documentation for educational, research and
 *       non-profit purposes, without fee, and without a written agreement is hereby granted, provided that the above copyright
 *       notice and the following three paragraphs appear in all copies.<br>
 *       <br>
 *       To request Permission to incorporate this software into commercial products contact Frederic Vernier, 19 butte aux
 *       cailles street, Paris, 75013, France. Tel: (+33) 871 747 387. eMail: Frederic.Vernier@laposte.net / Web site: http://vernier.frederic.free.fr
 *       <br>
 *       IN NO EVENT SHALL FREDERIC VERNIER BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 *       DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF FREDERIC
 *       VERNIER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.<br>
 *       <br>
 *       FREDERIC VERNIER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *       MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HERE UNDER IS ON AN "AS IS" BASIS, AND
 *       FREDERIC VERNIER HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.<br>
 *     </td>
 *   </tr>
 * </table>
 * <br>
 * <b>Project related :</b> FiCell, FieldExplorer<br>
 * <br>
 * <b>Dates:</b> <br>
 * <li> Creation : 15/07/2005<br>
 * <li> Format : 15/07/2005<br>
 * <li> Last Modif : 15/07/2005 <br>
 * <br>
 * <b>Bugs:</b> <br>
 * <li> <br>
 * <br>
 * <b>To Do:</b> <br>
 * <br>
 *
 * @author    Frederic Vernier, Frederic.Vernier@laposte.net
 * @version   1.5.2
 */

public class UnitPainterEvent
   extends AWTEvent
   implements Cloneable {

  //---------- MODIFIERS|-------------------- Type|----------------------------------------------- Name = Init value
  protected final static  javax.swing.text.html.parser.ParserDelegator  MAXIMUM_VARIABLE_SIZE_FOR_NAME  = null;

  // the data
  protected               Graphics                                      graphics;
  protected               double                                        value;
  protected               Color                                         color;
  protected               int                                           segmentIndex;
  protected               Rectangle                                     rectangle;


  /**
   * duplicate the object
   *
   * @return                                Description of the Return Value
   * @exception CloneNotSupportedException  Description of the Exception
   */
  @Override
  public Object clone() throws CloneNotSupportedException {
    return super.clone();
  }


  /**
   * Constructor
   *
   * @param source     Source of the event. Should be the BiSlider instance
   * @param graphics   The graphics object to draw with
   * @param value      The value to paint
   * @param index      The index of the segment
   * @param color      The color normally associated with these values
   * @param rectangle  The rectangle to fill with painting
   */
  public UnitPainterEvent(
    Object source,
    Graphics graphics,
    double value,
    int index,
    Color color,
    Rectangle rectangle) {

    super(source, 0);

    this.segmentIndex      = index;
    this.graphics          = graphics;
    this.value             = value;
    this.color             = color;
    this.rectangle         = rectangle;
  } // Constructor


  /**
   * @return   the Graphics to draw the rectangle with
   */
  public Graphics getGraphics() {
    return graphics;
  } // getGraphics()

  /**
   * @return the Index of the segment we are drawing
   */
  public int getSegmentIndex() {
    return segmentIndex;
  } // getSegmentIndex()


  /**
   * @return   the minimum value to paint
   */
  public double getValue() {
    return value;
  } // getMinimum()


  /**
   * @return   the default color to paint the rectangle
   */
  public Color getColor() {
    return color;
  } // getColor()


  /**
   * @return   the Rectangle, bounding box to paint something in
   */
  public Rectangle getRectangle() {
    return rectangle;
  } // getRectangle()



  /**
   * display the content of the color table
   *
   * @return   Description of the Return Value
   */
  @Override
  public String toString() {
    return "UnitPainterEvent@" + hashCode() + " to paint " + rectangle + " with " + color + " to represent " + value;
  } // toString()
} // ContentPainterEvent.java

