package com.visutools.nav.bislider;

import java.awt.*;
import java.util.ArrayList;

/**
 * The support object to store registered listeners and fire events. <br>
 * <br>
 *
 * <table border=1 width = "90%">
 *   <tr>
 *     <td>
 *       Copyright 1997-2011 Frederic Vernier. All Rights Reserved.<br>
 *       <br>
 *       Permission to use, copy, modify and distribute this software and its documentation for educational, research and
 *       non-profit purposes, without fee, and without a written agreement is hereby granted, provided that the above copyright
 *       notice and the following three paragraphs appear in all copies.<br>
 *       <br>
 *       To request Permission to incorporate this software into commercial products contact Frederic Vernier, 19 butte aux
 *       cailles street, Paris, 75013, France. Tel: (+33) 871 747 387. eMail: Frederic.Vernier@laposte.net / Web site: http://vernier.frederic.free.fr
 *       <br>
 *       IN NO EVENT SHALL FREDERIC VERNIER BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 *       DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF FREDERIC
 *       VERNIER HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.<br>
 *       <br>
 *       FREDERIC VERNIER SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *       MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HERE UNDER IS ON AN "AS IS" BASIS, AND
 *       FREDERIC VERNIER HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.<br>
 *     </td>
 *   </tr>
 * </table>
 * <br>
 * <b>Project related :</b> FiCell, FieldExplorer<br>
 * <br>
 * <b>Dates:</b> <br>
 * <li> Creation : 15/07/2005<br>
 * <li> Format : 15/07/2005<br>
 * <li> Last Modif : 15/07/2005<br>
 * <br>
 * <b>Bugs:</b> <br>
 *
 * <li> <br>
 * <br>
 * <b>To Do:</b> <br>
 * <br>
 *
 *
 * @author    Frederic Vernier, Frederic.Vernier@laposte.net
 * @version   1.5.2
 */

public class UnitPainterSupport implements java.io.Serializable {
  //---------- MODIFIERS|-------------------- Type|----------------------------------------------- Name = Init value
  protected final static  javax.swing.text.html.parser.ParserDelegator  MAXIMUM_VARIABLE_SIZE_FOR_NAME  = null;

  protected               ArrayList<UnitPainterListener>                UnitPainterListeners            = new ArrayList<UnitPainterListener>();

  final static            long                                          serialVersionUID                = -1497023291489642695L;


  /**
   * Adds a feature to the UnitPainterListener attribute of the UnitPainterSupport object
   *
   * @param l  The feature to be added to the UnitPainterListener attribute
   */
  public synchronized void addUnitPainterListener(UnitPainterListener l) {
    // add a listener if it is not already registered
    if (!UnitPainterListeners.contains(l)) {
      UnitPainterListeners.add(l);
    }
  } // addUnitPainterListener()


  /**
   * Description of the Method
   *
   * @param l  Description of the Parameter
   */
  public synchronized void removeUnitPainterListener(UnitPainterListener l) {
    // remove it if it is registered
    if (UnitPainterListeners.contains(l)) {
      UnitPainterListeners.remove(l);
    }
  } // removeUnitPainterListener()


  /**
   * Description of the Method
   */
  public synchronized void removeAllUnitPainterListener() {
    UnitPainterListeners.clear();
  } // removeUnitPainterListener()



  /**
   * return the number of listeners
   */
  public synchronized int getPainterListenerNumber() {
    return UnitPainterListeners.size();
  } // removeUnitPainterListener()


  /**
   * fire the event asynchronously
   *
   * @param source             Source of the event. Should be the BiSlider instance
   * @param graphics           The graphics object to draw with
   * @param value              The value to paint
   * @param index              The index of the segment
   * @param color              The color normally associated with these values
   * @param rectangle          The rectangle to fill with painting
   */
  public void fireAsyncPaintTick(
    final Object source,
    final Graphics graphics,
    final double value,
    final int index,
    final Color color,
    final Rectangle rectangle) {
    Thread  Thread1  =
      new Thread() {
        @Override
        public void run() {
          internFireAsyncPaintTick(source, graphics, value, index, color, rectangle);
        }
      };
    Thread1.start();
  } // fireAsyncPaintTick()


  /**
   * fire the event asynchronously
   *
   * @param source             Source of the event. Should be the BiSlider instance
   * @param graphics           The graphics object to draw with
   * @param value              The value to paint
   * @param color              The color normally associated with these values
   * @param rectangle          The rectangle to fill with painting
   */
  public void fireAsyncPaintValue(
    final Object source,
    final Graphics graphics,
    final double value,
    final Color color,
    final Rectangle rectangle) {
    Thread  Thread1  =
      new Thread() {
        @Override
        public void run() {
          internFireAsyncPaintValue(source, graphics, value, color, rectangle);
        }
      };
    Thread1.start();
  } // fireAsyncPaintValue()


 /**
   * fire the event asynchronously
   *
   * @param source             Source of the event. Should be the BiSlider
   * @param graphics           The graphics object to draw with
   * @param value              The value to paint
   * @param index              The index of the segment
   * @param color              The color normally associated with these values
   * @param rectangle          The rectangle to fill with painting
   */
  private void internFireAsyncPaintValue(
    final Object source,
    Graphics graphics,
    double value,
    Color color,
    Rectangle rectangle) {

    ArrayList<UnitPainterListener>  list;
    synchronized (this) {
      list = (ArrayList<UnitPainterListener>)UnitPainterListeners.clone();
    }

    // Fire the event to all listeners.
    int     count    = list.size();
    for (int i = 0; i < count; i++) {
      UnitPainterListener  listener  = list.get(i);
      listener.paintValue(new UnitPainterEvent(source, graphics, value, 0, color, rectangle));
    }
  } // internFireAsyncPaintValue()


  /**
   * fire the event asynchronously
   *
   * @param source             Source of the event. Should be the BiSlider
   * @param graphics           The graphics object to draw with
   * @param value              The value to paint
   * @param index              The index of the segment
   * @param color              The color normally associated with these values
   * @param rectangle          The rectangle to fill with painting
   */
  private void internFireAsyncPaintTick(
    final Object source,
    Graphics graphics,
    double value,
    int index,
    Color color,
    Rectangle rectangle) {

    ArrayList<UnitPainterListener>  list;
    synchronized (this) {
      list = (ArrayList<UnitPainterListener>)UnitPainterListeners.clone();
    }

    // Fire the event to all listeners.
    int     count    = list.size();
    for (int i = 0; i < count; i++) {
      UnitPainterListener  listener  = list.get(i);
      listener.paintTick(new UnitPainterEvent(source, graphics, value, index, color, rectangle));
    }
  } // internFireAsyncPaintTick()


  /**
   * fire the event
   *
   * @param source             Source of the event. Should be the BiSlider
   * @param graphics           The graphics object to draw with
   * @param value              The value to paint
   * @param index              The index of the segment
   * @param color              The color normally associated with these values
   * @param rectangle          The rectangle to fill with painting
   */
  public void firePaintTick(
    final Object source,
    Graphics graphics,
    double value,
    int index,
    Color color,
    Rectangle rectangle) {

    // Make a copy of the listener object vector so that
    // it cannot be changed while we are firing events.
    ArrayList<UnitPainterListener>  list;
    synchronized (this) {
      list = (ArrayList<UnitPainterListener>)UnitPainterListeners.clone();
    }

    // Fire the event to all listeners.
    int     count    = list.size();
    for (int i = 0; i < count; i++) {
      UnitPainterListener  listener  = (UnitPainterListener)list.get(i);
      listener.paintTick(new UnitPainterEvent(source, graphics, value, index, color, rectangle));
    }
  } // firePaintTick()


  /**
   * fire the event
   *
   * @param source             Source of the event. Should be the BiSlider
   * @param graphics           The graphics object to draw with
   * @param value              The value to paint
   * @param color              The color normally associated with these values
   * @param rectangle          The rectangle in which the other rectangle is
   */
  public void firePaintValue(
    final Object source,
    Graphics graphics,
    double value,
    Color color,
    Rectangle rectangle) {

    // Make a copy of the listener object vector so that
    // it cannot be changed while we are firing events.
    ArrayList<UnitPainterListener>  list;
    synchronized (this) {
      list = (ArrayList<UnitPainterListener>)UnitPainterListeners.clone();
    }

    // Fire the event to all listeners.
    int     count    = list.size();
    for (int i = 0; i < count; i++) {
      UnitPainterListener  listener  = (UnitPainterListener)list.get(i);
      listener.paintValue(new UnitPainterEvent(source, graphics, value, 0, color, rectangle));
    }
  } // firePaintValue()
}

